@@ -227,7 +227,7 @@ const (
 func addGlobalFlags(set *pflag.FlagSet) {
 	set.StringVar(&GlobalFlags.SelectedEnv, flagEnv, "", "set which env from the config file to use")
 	set.Var(&GlobalFlags.Vars, flagVar, "input variables")
-	set.StringVarP(&GlobalFlags.ConfigURL, flagConfig, "c", projectFileName, "select config (project) file using URL format")
+	set.StringVarP(&GlobalFlags.ConfigURL, flagConfig, "c", defaultConfigPath, "select config (project) file using URL format")
 }
 
 func addFlagAutoApprove(set *pflag.FlagSet, target *bool) {
@@ -440,7 +440,11 @@ func stateReader(ctx context.Context, config *stateReaderConfig) (*stateReadClos
 			if err != nil {
 				return nil, err
 			}
-			return &stateReadCloser{StateReader: sr}, nil
+			rc := &stateReadCloser{StateReader: sr}
+			if config.dev != nil && config.dev.URL.Schema != "" {
+				rc.schema = config.dev.URL.Schema
+			}
+			return rc, nil
 		}
 		// All other schemes are database (or docker) connections.
 		c, err := sqlclient.Open(ctx, config.urls[0]) // call to selectScheme already checks for len > 0
@@ -541,10 +545,18 @@ func sqlStateReader(ctx context.Context, config *stateReaderConfig, urls []*url.
 		if err != nil {
 			return nil, err
 		}
-		dir = &validMemDir{}
+		dir = &migrate.MemDir{}
 		if err := dir.WriteFile(fi.Name(), b); err != nil {
 			return nil, err
 		}
+		// Create a checksum file to bypass the checksum check.
+		sum, err := dir.Checksum()
+		if err != nil {
+			return nil, err
+		}
+		if err = migrate.WriteSumFile(dir, sum); err != nil {
+			return nil, err
+		}
 	// A migration directory.
 	default:
 		if dir, err = dirURL(urls[0], false); err != nil {
@@ -583,11 +595,6 @@ func (sr *stateReadCloser) Close() {
 	}
 }
 
-// validMemDir will not throw an error when put into migrate.Validate.
-type validMemDir struct{ migrate.MemDir }
-
-func (d *validMemDir) Validate() error { return nil }
-
 const (
 	extHCL = ".hcl"
 	extSQL = ".sql"
