@@ -22,6 +22,7 @@ import (
 	"text/template/parse"
 	"time"
 
+	"ariga.io/atlas/cmd/atlas/internal/cloudapi"
 	"ariga.io/atlas/cmd/atlas/internal/cmdext"
 	"ariga.io/atlas/cmd/atlas/internal/cmdlog"
 	"ariga.io/atlas/cmd/atlas/internal/lint"
@@ -34,6 +35,7 @@ import (
 	"ariga.io/atlas/sql/sqlclient"
 	"ariga.io/atlas/sql/sqltool"
 
+	"github.com/google/uuid"
 	"github.com/hashicorp/hcl/v2"
 	"github.com/hashicorp/hcl/v2/hclparse"
 	"github.com/hashicorp/hcl/v2/hclsyntax"
@@ -80,6 +82,19 @@ type migrateApplyFlags struct {
 	txMode          string // (none, file, all)
 }
 
+func (f *migrateApplyFlags) migrateOptions() (opts []migrate.ExecutorOption) {
+	if f.allowDirty {
+		opts = append(opts, migrate.WithAllowDirty(true))
+	}
+	if v := f.baselineVersion; v != "" {
+		opts = append(opts, migrate.WithBaselineVersion(v))
+	}
+	if v := f.fromVersion; v != "" {
+		opts = append(opts, migrate.WithFromVersion(v))
+	}
+	return
+}
+
 func migrateApplyCmd() *cobra.Command {
 	var (
 		flags migrateApplyFlags
@@ -99,11 +114,24 @@ If run with the "--dry-run" flag, atlas will not execute any SQL.`,
   atlas migrate apply --env dev 1
   atlas migrate apply --dry-run --env dev 1`,
 			Args: cobra.MaximumNArgs(1),
-			RunE: func(cmd *cobra.Command, args []string) error {
-				if GlobalFlags.SelectedEnv == "" {
-					return migrateApplyRun(cmd, args, flags)
+			RunE: func(cmd *cobra.Command, args []string) (cmdErr error) {
+				switch {
+				case GlobalFlags.SelectedEnv == "":
+					if err := migrateFlagsFromConfig(cmd); err != nil {
+						return err
+					}
+					return migrateApplyRun(cmd, args, flags, &MigrateReport{}) // nop reporter
+				default:
+					project, envs, err := EnvByName(GlobalFlags.SelectedEnv, WithInput(GlobalFlags.Vars))
+					if err != nil {
+						return err
+					}
+					set := NewReportProvider(project, envs)
+					defer func() { set.Flush(cmd, cmdErr) }()
+					return cmdEnvsRun(envs, setMigrateEnvFlags, cmd, func(env *Env) error {
+						return migrateApplyRun(cmd, args, flags, set.ReportFor(flags, env))
+					})
 				}
-				return cmdEnvsRun(migrateApplyRun, setMigrateEnvFlags, cmd, args, &flags)
 			},
 		}
 	)
@@ -125,10 +153,10 @@ If run with the "--dry-run" flag, atlas will not execute any SQL.`,
 }
 
 // migrateApplyCmd represents the 'atlas migrate apply' subcommand.
-func migrateApplyRun(cmd *cobra.Command, args []string, flags migrateApplyFlags) error {
+func migrateApplyRun(cmd *cobra.Command, args []string, flags migrateApplyFlags, mr *MigrateReport) (err error) {
 	var (
 		count int
-		err   error
+		ctx   = cmd.Context()
 	)
 	if len(args) > 0 {
 		if count, err = strconv.Atoi(args[0]); err != nil {
@@ -151,14 +179,16 @@ func migrateApplyRun(cmd *cobra.Command, args []string, flags migrateApplyFlags)
 	if flags.url == "" {
 		return errors.New(`required flag "url" not set`)
 	}
-	client, err := sqlclient.Open(cmd.Context(), flags.url)
+	client, err := sqlclient.Open(ctx, flags.url)
 	if err != nil {
 		return err
 	}
 	defer client.Close()
+	// Prevent usage printing after input validation.
+	cmd.SilenceUsage = true
 	// Acquire a lock.
 	if l, ok := client.Driver.(schema.Locker); ok {
-		unlock, err := l.Lock(cmd.Context(), applyLockValue, flags.lockTimeout)
+		unlock, err := l.Lock(ctx, applyLockValue, flags.lockTimeout)
 		if err != nil {
 			return fmt.Errorf("acquiring database lock: %w", err)
 		}
@@ -169,51 +199,47 @@ func migrateApplyRun(cmd *cobra.Command, args []string, flags migrateApplyFlags)
 		return err
 	}
 	var rrw migrate.RevisionReadWriter
-	rrw, err = entRevisions(cmd.Context(), client, flags.revisionSchema)
-	if err != nil {
+	if rrw, err = entRevisions(ctx, client, flags.revisionSchema); err != nil {
 		return err
 	}
-	if err := rrw.(*cmdmigrate.EntRevisions).Migrate(cmd.Context()); err != nil {
+	if err := rrw.(*cmdmigrate.EntRevisions).Migrate(ctx); err != nil {
+		return err
+	}
+	// Setup reporting info.
+	report := cmdlog.NewMigrateApply(client, migrationDir)
+	mr.Init(client, report, rrw.(*cmdmigrate.EntRevisions))
+	// If cloud reporting is enabled, and we cannot obtain the current
+	// target identifier, abort and report it to the user.
+	if err := mr.RecordTargetID(cmd.Context()); err != nil {
 		return err
 	}
 	// Determine pending files.
-	opts := []migrate.ExecutorOption{
-		migrate.WithOperatorVersion(operatorVersion()),
-	}
-	if flags.allowDirty {
-		opts = append(opts, migrate.WithAllowDirty(true))
-	}
-	if v := flags.baselineVersion; v != "" {
-		opts = append(opts, migrate.WithBaselineVersion(v))
-	}
-	if v := flags.fromVersion; v != "" {
-		opts = append(opts, migrate.WithFromVersion(v))
-	}
-	report := cmdlog.NewMigrateApply(client, migrationDir)
+	opts := append(flags.migrateOptions(), migrate.WithOperatorVersion(operatorVersion()), migrate.WithLogger(report))
 	ex, err := migrate.NewExecutor(client.Driver, migrationDir, rrw, opts...)
 	if err != nil {
 		return err
 	}
-	pending, err := ex.Pending(cmd.Context())
+	pending, err := ex.Pending(ctx)
 	if err != nil && !errors.Is(err, migrate.ErrNoPendingFiles) {
 		return err
 	}
-	if errors.Is(err, migrate.ErrNoPendingFiles) {
-		return reportApply(cmd, flags, report)
+	noPending := errors.Is(err, migrate.ErrNoPendingFiles)
+	// Get the pending files before obtaining applied revisions,
+	// as the Executor may write a baseline revision in the table.
+	applied, err := rrw.ReadRevisions(ctx)
+	if err != nil {
+		return err
+	}
+	if noPending {
+		migrate.LogNoPendingFiles(report, applied)
+		return mr.Done(cmd, flags)
 	}
 	if l := len(pending); count == 0 || count >= l {
 		// Cannot apply more than len(pending) migration files.
 		count = l
 	}
 	pending = pending[:count]
-	applied, err := rrw.ReadRevisions(cmd.Context())
-	if err != nil {
-		return err
-	}
-	opts = append(opts, migrate.WithLogger(report))
-	if err := migrate.LogIntro(report, applied, pending); err != nil {
-		return err
-	}
+	migrate.LogIntro(report, applied, pending)
 	var (
 		mux = tx{
 			dryRun: flags.dryRun,
@@ -225,40 +251,246 @@ func migrateApplyRun(cmd *cobra.Command, args []string, flags migrateApplyFlags)
 		drv migrate.Driver
 	)
 	for _, f := range pending {
-		drv, rrw, err = mux.driverFor(cmd.Context(), f)
-		if err != nil {
-			return err
+		if drv, rrw, err = mux.driverFor(ctx, f); err != nil {
+			break
 		}
-		ex, err = migrate.NewExecutor(drv, migrationDir, rrw, opts...)
-		if err != nil {
-			return err
+		if ex, err = migrate.NewExecutor(drv, migrationDir, rrw, opts...); err != nil {
+			return fmt.Errorf("unexpected exectuor creation error: %w", err)
 		}
-		if err = mux.mayRollback(ex.Execute(cmd.Context(), f)); err != nil {
-			report.Error = err.Error()
+		if err = mux.mayRollback(ex.Execute(ctx, f)); err != nil {
 			break
 		}
 		if err = mux.mayCommit(); err != nil {
-			report.Error = err.Error()
 			break
 		}
 	}
 	if err == nil {
-		if err = mux.commit(); err != nil {
-			report.Error = err.Error()
-		} else {
+		if err = mux.commit(); err == nil {
 			report.Log(migrate.LogDone{})
 		}
 	}
-	if err2 := reportApply(cmd, flags, report); err2 != nil {
-		if err != nil {
-			err2 = fmt.Errorf("%w: %v", err, err2)
-		}
-		err = err2
+	if err != nil {
+		report.Error = err.Error()
 	}
+	return errors.Join(err, mr.Done(cmd, flags))
+}
+
+type (
+	// MigrateReport responsible for reporting 'migrate apply' reports.
+	MigrateReport struct {
+		id     string // target id
+		env    *Env   // nil, if no env set
+		client *sqlclient.Client
+		log    *cmdlog.MigrateApply
+		rrw    *cmdmigrate.EntRevisions
+		done   func(*cloudapi.ReportMigrationInput)
+	}
+	// MigrateReportSet is a set of reports.
+	MigrateReportSet struct {
+		cloudapi.ReportMigrationSetInput
+		client *cloudapi.Client
+		done   int // number of done migrations
+	}
+)
+
+// NewReportProvider returns a new ReporterProvider.
+func NewReportProvider(project *Project, envs []*Env) *MigrateReportSet {
+	s := &MigrateReportSet{
+		client: project.cfg.Client,
+		ReportMigrationSetInput: cloudapi.ReportMigrationSetInput{
+			ID:        uuid.NewString(),
+			StartTime: time.Now(),
+			Planned:   len(envs),
+		},
+	}
+	s.Step("Start migration for %d targets", len(envs))
+	for _, e := range envs {
+		s.StepLog(s.RedactedURL(e.URL))
+	}
+	return s
+}
+
+// RedactedURL returns the redacted URL of the given environment at index i.
+func (*MigrateReportSet) RedactedURL(u string) string {
+	u, err := cloudapi.RedactedURL(u)
+	if err != nil {
+		return fmt.Sprintf("Error: redacting URL: %v", err)
+	}
+	return u
+}
+
+// Step starts a new reporting step.
+func (s *MigrateReportSet) Step(format string, args ...interface{}) {
+	if len(s.Log) > 0 && s.Log[len(s.Log)-1].EndTime.IsZero() {
+		s.Log[len(s.Log)-1].EndTime = time.Now()
+	}
+	s.Log = append(s.Log, cloudapi.ReportStep{
+		StartTime: time.Now(),
+		Text:      fmt.Sprintf(format, args...),
+	})
+}
+
+// StepLog logs a line to the current reporting step.
+func (s *MigrateReportSet) StepLog(format string, args ...interface{}) {
+	if len(s.Log) == 0 {
+		s.Step("Unnamed step") // Unexpected.
+	}
+	s.Log[len(s.Log)-1].Log = append(s.Log[len(s.Log)-1].Log, cloudapi.ReportStepLog{
+		Text: fmt.Sprintf(format, args...),
+	})
+}
+
+// StepLogError logs a line to the current reporting step.
+func (s *MigrateReportSet) StepLogError(text string) {
+	if !strings.HasPrefix(text, "Error") {
+		text = "Error: " + text
+	}
+	s.StepLog(text)
+	s.Error = &text
+	s.Log[len(s.Log)-1].Error = true
+}
+
+// ReportFor returns a new MigrateReport for the given environment.
+func (s *MigrateReportSet) ReportFor(flags migrateApplyFlags, e *Env) *MigrateReport {
+	s.Step("Run migration: %d", s.done+1)
+	s.StepLog("Target URL: %s", s.RedactedURL(e.URL))
+	s.StepLog("Migration directory: %s", s.RedactedURL(flags.dirURL))
+	return &MigrateReport{
+		env: e,
+		done: func(r *cloudapi.ReportMigrationInput) {
+			s.done++
+			s.Log[len(s.Log)-1].EndTime = time.Now()
+			if r.Error != nil && *r.Error != "" {
+				s.StepLogError(*r.Error)
+			}
+			s.Completed = append(s.Completed, *r)
+		},
+	}
+}
+
+// Flush report the migration deployment to the cloud.
+// The current implementation is simplistic and sends each
+// report separately without marking them as part of a group.
+//
+// Note that reporting errors are logged, but not cause Atlas to fail.
+func (s *MigrateReportSet) Flush(cmd *cobra.Command, cmdErr error) {
+	if cmdErr != nil && s.Error == nil {
+		s.StepLogError(cmdErr.Error())
+	}
+	var err error
+	switch {
+	// Skip reporting if set is empty,
+	// or there is no cloud connectivity.
+	case s.Planned == 0, s.client == nil:
+		return
+	// Single migration that was completed.
+	case s.Planned == 1 && len(s.Completed) == 1:
+		err = s.client.ReportMigration(cmd.Context(), s.Completed[0])
+	// Single migration that failed to start.
+	case s.Planned == 1 && len(s.Completed) == 0:
+		s.EndTime = time.Now()
+		err = s.client.ReportMigrationSet(cmd.Context(), s.ReportMigrationSetInput)
+	// Multi environment migration (e.g., multi-tenancy).
+	case s.Planned > 1:
+		s.EndTime = time.Now()
+		err = s.client.ReportMigrationSet(cmd.Context(), s.ReportMigrationSetInput)
+	}
+	if err != nil {
+		txt := fmt.Sprintf("Error: %s", strings.TrimRight(err.Error(), "\n"))
+		// Ensure errors are printed in new lines.
+		if cmd.Flags().Changed(flagFormat) {
+			txt = "\n" + txt
+		}
+		cmd.PrintErrln(txt)
+	}
+}
+
+// Init the report if the necessary dependencies.
+func (r *MigrateReport) Init(c *sqlclient.Client, l *cmdlog.MigrateApply, rrw *cmdmigrate.EntRevisions) {
+	r.client, r.log, r.rrw = c, l, rrw
+}
+
+// RecordTargetID asks the revisions-table to allow or provide
+// the target identifier if cloud reporting is enabled.
+func (r *MigrateReport) RecordTargetID(ctx context.Context) error {
+	if r.CloudEnabled() {
+		id, err := r.rrw.ID(ctx, operatorVersion())
+		if err != nil {
+			return err
+		}
+		r.id = id
+	}
+	return nil
+}
+
+// Done closes and flushes this report.
+func (r *MigrateReport) Done(cmd *cobra.Command, flags migrateApplyFlags) error {
+	if !r.CloudEnabled() {
+		return logApply(cmd, cmd.OutOrStdout(), flags, r.log)
+	}
+	var (
+		ver  string
+		clog bytes.Buffer
+		err  = logApply(cmd, io.MultiWriter(cmd.OutOrStdout(), &clog), flags, r.log)
+	)
+	switch rev, err1 := r.rrw.CurrentRevision(cmd.Context()); {
+	case ent.IsNotFound(err1):
+	case err1 != nil:
+		return errors.Join(err, err1)
+	default:
+		ver = rev.Version
+	}
+	dirName := flags.dirURL
+	if u, err := url.Parse(flags.dirURL); err == nil {
+		dirName = filepath.Join(u.Host, u.Path)
+	}
+	r.done(&cloudapi.ReportMigrationInput{
+		ProjectName:  r.env.cfg.Project,
+		EnvName:      r.env.Name,
+		DirName:      dirName,
+		AtlasVersion: operatorVersion(),
+		Target: cloudapi.DeployedTargetInput{
+			ID:     r.id,
+			Schema: r.client.URL.Schema,
+			URL:    r.client.URL.Redacted(),
+		},
+		StartTime:      r.log.Start,
+		EndTime:        r.log.End,
+		FromVersion:    r.log.Current,
+		ToVersion:      r.log.Target,
+		CurrentVersion: ver,
+		Error: func() *string {
+			if r.log.Error != "" {
+				return &r.log.Error
+			}
+			return nil
+		}(),
+		Files: func() []cloudapi.DeployedFileInput {
+			files := make([]cloudapi.DeployedFileInput, len(r.log.Applied))
+			for i, f := range r.log.Applied {
+				files[i] = cloudapi.DeployedFileInput{
+					Name:      f.Name(),
+					Content:   string(f.Bytes()),
+					StartTime: f.Start,
+					EndTime:   f.End,
+					Skipped:   f.Skipped,
+					Applied:   len(f.Applied),
+					Error:     (*cloudapi.StmtErrorInput)(f.Error),
+				}
+			}
+			return files
+		}(),
+		Log: clog.String(),
+	})
 	return err
 }
 
-func reportApply(cmd *cobra.Command, flags migrateApplyFlags, r *cmdlog.MigrateApply) error {
+// CloudEnabled reports if cloud reporting is enabled.
+func (r *MigrateReport) CloudEnabled() bool {
+	return r.env != nil && r.env.cfg != nil && r.env.cfg.Client != nil && r.env.cfg.Project != ""
+}
+
+func logApply(cmd *cobra.Command, w io.Writer, flags migrateApplyFlags, r *cmdlog.MigrateApply) error {
 	var (
 		err error
 		f   = cmdlog.MigrateApplyTemplate
@@ -269,7 +501,7 @@ func reportApply(cmd *cobra.Command, flags migrateApplyFlags, r *cmdlog.MigrateA
 			return fmt.Errorf("parse format: %w", err)
 		}
 	}
-	if err = f.Execute(cmd.OutOrStdout(), r); err != nil {
+	if err = f.Execute(w, r); err != nil {
 		return fmt.Errorf("execute log template: %w", err)
 	}
 	// In case a custom logging was configured, avoid reporting errors twice.
@@ -306,7 +538,7 @@ directory state to the desired schema. The desired state can be another connecte
   atlas migrate diff --env dev --format '{{ sql . "  " }}'`,
 			Args: cobra.MaximumNArgs(1),
 			PreRunE: func(cmd *cobra.Command, args []string) error {
-				if err := migrateFlagsFromEnv(cmd); err != nil {
+				if err := migrateFlagsFromConfig(cmd); err != nil {
 					return err
 				}
 				if err := dirFormatBC(flags.dirFormat, &flags.dirURL); err != nil {
@@ -315,7 +547,11 @@ directory state to the desired schema. The desired state can be another connecte
 				return checkDir(cmd, flags.dirURL, true)
 			},
 			RunE: func(cmd *cobra.Command, args []string) error {
-				return migrateDiffRun(cmd, args, flags)
+				env, err := selectEnv(cmd)
+				if err != nil {
+					return err
+				}
+				return migrateDiffRun(cmd, args, flags, env)
 			},
 		}
 	)
@@ -335,7 +571,7 @@ directory state to the desired schema. The desired state can be another connecte
 	return cmd
 }
 
-func migrateDiffRun(cmd *cobra.Command, args []string, flags migrateDiffFlags) error {
+func migrateDiffRun(cmd *cobra.Command, args []string, flags migrateDiffFlags, env *Env) error {
 	ctx := cmd.Context()
 	dev, err := sqlclient.Open(ctx, flags.devURL)
 	if err != nil {
@@ -378,11 +614,12 @@ func migrateDiffRun(cmd *cobra.Command, args []string, flags migrateDiffFlags) e
 	// 'migrate diff' handling, offload it the work.
 	if d, ok := cmdext.States.Differ(flags.desiredURLs); ok {
 		err := d.MigrateDiff(ctx, &cmdext.MigrateDiffOptions{
-			To:     flags.desiredURLs,
-			Name:   name,
-			Indent: indent,
-			Dir:    dir,
-			Dev:    dev,
+			To:      flags.desiredURLs,
+			Name:    name,
+			Indent:  indent,
+			Dir:     dir,
+			Dev:     dev,
+			Options: env.DiffOptions(),
 		})
 		return maskNoPlan(cmd, err)
 	}
@@ -398,7 +635,11 @@ func migrateDiffRun(cmd *cobra.Command, args []string, flags migrateDiffFlags) e
 		return err
 	}
 	defer desired.Close()
-	opts := []migrate.PlannerOption{migrate.PlanFormat(f), migrate.PlanWithIndent(indent)}
+	opts := []migrate.PlannerOption{
+		migrate.PlanFormat(f),
+		migrate.PlanWithIndent(indent),
+		migrate.PlanWithDiffOptions(env.DiffOptions()...),
+	}
 	if dev.URL.Schema != "" {
 		// Disable tables qualifier in schema-mode.
 		opts = append(opts, migrate.PlanWithSchemaQualifier(flags.qualifier))
@@ -493,7 +734,7 @@ func migrateHashCmd() *cobra.Command {
 This command should be used whenever a manual change in the migration directory was made.`,
 			Example: `  atlas migrate hash`,
 			PreRunE: func(cmd *cobra.Command, args []string) error {
-				if err := migrateFlagsFromEnv(cmd); err != nil {
+				if err := migrateFlagsFromConfig(cmd); err != nil {
 					return err
 				}
 				return dirFormatBC(flags.dirFormat, &flags.dirURL)
@@ -531,7 +772,7 @@ func migrateImportCmd() *cobra.Command {
 			// Validate the source directory. Consider a directory with no sum file
 			// valid, since it might be an import from an existing project.
 			PreRunE: func(cmd *cobra.Command, _ []string) error {
-				if err := migrateFlagsFromEnv(cmd); err != nil {
+				if err := migrateFlagsFromConfig(cmd); err != nil {
 					return err
 				}
 				if err := dirFormatBC(flags.dirFormat, &flags.fromURL); err != nil {
@@ -596,8 +837,8 @@ func migrateImportRun(cmd *cobra.Command, _ []string, flags migrateImportFlags)
 	if _, ok := src.(*sqltool.FlywayDir); ok {
 		sqltool.SetRepeatableVersion(ff)
 	}
-	// Extract the statements for each of the migration files, add them to a plan to format with the
-	// migrate.DefaultFormatter.
+	// Extract the statements for each of the migration files,
+	// add them to a plan to format with the DefaultFormatter.
 	for _, f := range ff {
 		stmts, err := f.StmtDecls()
 		if err != nil {
@@ -657,7 +898,7 @@ func migrateLintCmd() *cobra.Command {
   atlas migrate lint --dir file:///path/to/migration/directory --dev-url mysql://root:pass@localhost:3306 --git-base master
   atlas migrate lint --dir file:///path/to/migration/directory --dev-url mysql://root:pass@localhost:3306 --format '{{ json .Files }}'`,
 			PreRunE: func(cmd *cobra.Command, args []string) error {
-				if err := migrateFlagsFromEnv(cmd); err != nil {
+				if err := migrateFlagsFromConfig(cmd); err != nil {
 					return err
 				}
 				return dirFormatBC(flags.dirFormat, &flags.dirURL)
@@ -717,7 +958,7 @@ func migrateLintRun(cmd *cobra.Command, _ []string, flags migrateLintFlags) erro
 			return fmt.Errorf("parse format: %w", err)
 		}
 	}
-	env, err := selectEnv(GlobalFlags.SelectedEnv)
+	env, err := selectEnv(cmd)
 	if err != nil {
 		return err
 	}
@@ -759,7 +1000,7 @@ func migrateNewCmd() *cobra.Command {
 			Example: `  atlas migrate new my-new-migration`,
 			Args:    cobra.MaximumNArgs(1),
 			PreRunE: func(cmd *cobra.Command, _ []string) error {
-				if err := migrateFlagsFromEnv(cmd); err != nil {
+				if err := migrateFlagsFromConfig(cmd); err != nil {
 					return err
 				}
 				if err := dirFormatBC(flags.dirFormat, &flags.dirURL); err != nil {
@@ -821,7 +1062,7 @@ to be applied. This command is usually used after manually making changes to the
   atlas migrate set --env local
   atlas migrate set 1.2.4 --url mysql://user:pass@localhost:3306/my_db --revision-schema my_revisions`,
 			PreRunE: func(cmd *cobra.Command, _ []string) error {
-				if err := migrateFlagsFromEnv(cmd); err != nil {
+				if err := migrateFlagsFromConfig(cmd); err != nil {
 					return err
 				}
 				if err := dirFormatBC(flags.dirFormat, &flags.dirURL); err != nil {
@@ -1016,7 +1257,7 @@ func migrateStatusCmd() *cobra.Command {
 			Example: `  atlas migrate status --url mysql://user:pass@localhost:3306/
   atlas migrate status --url mysql://user:pass@localhost:3306/ --dir file:///path/to/migration/directory`,
 			PreRunE: func(cmd *cobra.Command, _ []string) error {
-				if err := migrateFlagsFromEnv(cmd); err != nil {
+				if err := migrateFlagsFromConfig(cmd); err != nil {
 					return err
 				}
 				if err := dirFormatBC(flags.dirFormat, &flags.dirURL); err != nil {
@@ -1090,7 +1331,7 @@ files are executed on the connected database in order to validate SQL semantics.
   atlas migrate validate --dir file:///path/to/migration/directory --dev-url mysql://user:pass@localhost:3306/dev
   atlas migrate validate --env dev`,
 			PreRunE: func(cmd *cobra.Command, _ []string) error {
-				if err := migrateFlagsFromEnv(cmd); err != nil {
+				if err := migrateFlagsFromConfig(cmd); err != nil {
 					return err
 				}
 				if err := dirFormatBC(flags.dirFormat, &flags.dirURL); err != nil {
@@ -1542,12 +1783,12 @@ func formatter(u *url.URL) (migrate.Formatter, error) {
 	}
 }
 
-func migrateFlagsFromEnv(cmd *cobra.Command) error {
-	activeEnv, err := selectEnv(GlobalFlags.SelectedEnv)
+func migrateFlagsFromConfig(cmd *cobra.Command) error {
+	env, err := selectEnv(cmd)
 	if err != nil {
 		return err
 	}
-	return setMigrateEnvFlags(cmd, activeEnv)
+	return setMigrateEnvFlags(cmd, env)
 }
 
 func setMigrateEnvFlags(cmd *cobra.Command, env *Env) error {
@@ -1614,6 +1855,9 @@ func setMigrateEnvFlags(cmd *cobra.Command, env *Env) error {
 		return err
 	}
 	for i, s := range srcs {
+		if isURL(s) {
+			continue
+		}
 		if s, err = filepath.Abs(s); err != nil {
 			return fmt.Errorf("finding abs path to source: %q: %w", s, err)
 		}
@@ -1628,16 +1872,20 @@ func setMigrateEnvFlags(cmd *cobra.Command, env *Env) error {
 	return nil
 }
 
+// isURL returns true if the given string
+// is an Atlas URL with a scheme.
+func isURL(s string) bool {
+	u, err := url.Parse(s)
+	return err == nil && u.Scheme != ""
+}
+
 // cmdEnvsRun executes a given command on each of the configured environment.
-func cmdEnvsRun[F any](
-	run func(*cobra.Command, []string, F) error,
-	set func(*cobra.Command, *Env) error,
-	cmd *cobra.Command, args []string, flags *F,
+func cmdEnvsRun(
+	envs []*Env,
+	setFlags func(*cobra.Command, *Env) error,
+	cmd *cobra.Command,
+	runCmd func(*Env) error,
 ) error {
-	envs, err := LoadEnv(GlobalFlags.SelectedEnv, WithInput(GlobalFlags.Vars))
-	if err != nil {
-		return err
-	}
 	var (
 		w     bytes.Buffer
 		out   = cmd.OutOrStdout()
@@ -1646,10 +1894,10 @@ func cmdEnvsRun[F any](
 	cmd.SetOut(io.MultiWriter(out, &w))
 	defer cmd.SetOut(out)
 	for i, e := range envs {
-		if err := set(cmd, e); err != nil {
+		if err := setFlags(cmd, e); err != nil {
 			return err
 		}
-		if err := run(cmd, args, *flags); err != nil {
+		if err := runCmd(e); err != nil {
 			return err
 		}
 		b := bytes.TrimLeft(w.Bytes(), " \t\r")
