@@ -7,10 +7,13 @@ package cmdapi
 import (
 	"context"
 	"database/sql"
+	"encoding/base64"
 	"encoding/json"
 	"errors"
 	"fmt"
 	"io"
+	"net/http"
+	"net/http/httptest"
 	"net/url"
 	"os"
 	"os/exec"
@@ -20,6 +23,7 @@ import (
 	"testing"
 	"time"
 
+	"ariga.io/atlas/cmd/atlas/internal/cloudapi"
 	"ariga.io/atlas/cmd/atlas/internal/cmdlog"
 	migrate2 "ariga.io/atlas/cmd/atlas/internal/migrate"
 	"ariga.io/atlas/sql/migrate"
@@ -28,7 +32,9 @@ import (
 	"ariga.io/atlas/sql/sqlite"
 	_ "ariga.io/atlas/sql/sqlite"
 	_ "ariga.io/atlas/sql/sqlite/sqlitecheck"
+
 	"github.com/fatih/color"
+	"github.com/google/uuid"
 	_ "github.com/mattn/go-sqlite3"
 	"github.com/stretchr/testify/require"
 )
@@ -436,6 +442,12 @@ variable "url" {
   type = string
 }
 
+locals {
+  string = "%test"
+  bool   = true
+  int    = 1
+}
+
 data "sql" "tenants" {
   url   = var.url
   query = <<EOS
@@ -443,7 +455,7 @@ SELECT name FROM tenants
 	WHERE mode LIKE ? AND active = ? AND created = ?
 EOS
   # Pass all types of arguments.
-  args  = ["%test", true, 1]
+  args  = [local.string, local.bool, local.int]
 }
 
 env "local" {
@@ -508,14 +520,12 @@ env "local" {
 				Tenant string
 				cmdlog.MigrateApply
 			}
-			fmt.Println(s)
 			require.NoError(t, json.Unmarshal([]byte(s), &r))
 			require.Empty(t, r.Pending)
 			require.Empty(t, r.Applied)
 			require.NotEmpty(t, r.Tenant)
 			require.Equal(t, "sqlite3", r.Driver)
 		}
-
 		_, err = db.Exec("INSERT INTO `tenants` (`name`) VALUES (NULL)")
 		require.NoError(t, err)
 		_, err = runCmd(
@@ -687,13 +697,16 @@ func TestMigrate_ApplyTxModeDirective(t *testing.T) {
 	)
 	require.EqualError(t, err, `cannot set txmode directive to "none" in "20220925094021_second.sql" when txmode "all" is set globally`)
 
-	_, err = runCmd(
+	s, err := runCmd(
 		migrateApplyCmd(),
 		"--dir", "file://testdata/sqlitetx4",
 		"--url", "sqlite://txmode?mode=memory&_fk=1",
 		"--tx-mode", txModeAll,
+		"--log", "{{ .Error }}",
 	)
 	require.EqualError(t, err, `unknown txmode "unknown" found in file directive "20220925094021_second.sql"`)
+	// Errors should be attached to the report.
+	require.Equal(t, s, `unknown txmode "unknown" found in file directive "20220925094021_second.sql"`)
 }
 
 func TestMigrate_ApplyBaseline(t *testing.T) {
@@ -774,6 +787,338 @@ env "local" {
 	})
 }
 
+func TestMigrate_ApplyCloudReport(t *testing.T) {
+	var (
+		dir     migrate.MemDir
+		status  int
+		report  cloudapi.ReportMigrationInput
+		reports cloudapi.ReportMigrationSetInput
+		srv     = httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+			var m struct {
+				Query     string `json:"query"`
+				Variables struct {
+					Input json.RawMessage `json:"input"`
+				} `json:"variables"`
+			}
+			require.NoError(t, json.NewDecoder(r.Body).Decode(&m))
+			switch {
+			case strings.Contains(m.Query, "query"):
+				// Checksum before archiving.
+				hf, err := dir.Checksum()
+				require.NoError(t, err)
+				ht, err := hf.MarshalText()
+				require.NoError(t, err)
+				require.NoError(t, dir.WriteFile(migrate.HashFileName, ht))
+				// Archive and send.
+				arc, err := migrate.ArchiveDir(&dir)
+				require.NoError(t, err)
+				fmt.Fprintf(w, `{"data":{"dir":{"content":%q}}}`, base64.StdEncoding.EncodeToString(arc))
+			case strings.Contains(m.Query, "mutation") && strings.Contains(m.Query, "ReportMigrationSet"):
+				if status != 0 {
+					w.WriteHeader(status)
+				}
+				require.NoError(t, json.Unmarshal(m.Variables.Input, &reports))
+			case strings.Contains(m.Query, "mutation") && strings.Contains(m.Query, "ReportMigration"):
+				if status != 0 {
+					w.WriteHeader(status)
+				}
+				require.NoError(t, json.Unmarshal(m.Variables.Input, &report))
+			default:
+				t.Fatalf("unexpected query: %s", m.Query)
+			}
+		}))
+		h = `
+variable "cloud_url" {
+  type = string
+}
+
+atlas {
+  cloud {
+    token   = "token"
+    url     = var.cloud_url
+    project = "example"
+  }
+}
+
+data "remote_dir" "migrations" {
+  name = "migrations/v1/mysql"
+}
+
+env {
+  name = atlas.env
+  migration {
+    dir = data.remote_dir.migrations.url
+  }
+}
+`
+		u    = openSQLite(t, "")
+		p    = t.TempDir()
+		path = filepath.Join(p, "atlas.hcl")
+	)
+	t.Cleanup(srv.Close)
+	require.NoError(t, os.WriteFile(path, []byte(h), 0600))
+
+	t.Run("NoPendingFiles", func(t *testing.T) {
+		cmd := migrateCmd()
+		cmd.AddCommand(migrateApplyCmd())
+		s, err := runCmd(
+			cmd, "apply",
+			"-c", "file://"+path,
+			"--env", "local",
+			"--url", u,
+			"--var", "cloud_url="+srv.URL,
+		)
+		require.NoError(t, err)
+		require.Equal(t, "No migration files to execute\n", s)
+		require.NotEmpty(t, report.Target.ID)
+		_, err = uuid.Parse(report.Target.ID)
+		require.NoError(t, err, "target id is not a valid uuid")
+		require.False(t, report.StartTime.IsZero())
+		require.False(t, report.EndTime.IsZero())
+		require.Equal(t, cloudapi.ReportMigrationInput{
+			ProjectName:  "example",
+			DirName:      "migrations/v1/mysql",
+			EnvName:      "local",
+			AtlasVersion: "Atlas CLI - development",
+			StartTime:    report.StartTime,
+			EndTime:      report.EndTime,
+			Files:        []cloudapi.DeployedFileInput{},
+			Target: cloudapi.DeployedTargetInput{
+				ID:     report.Target.ID, // generated uuid
+				Schema: "main",
+				URL:    u,
+			},
+			Log: "No migration files to execute\n",
+		}, report)
+	})
+
+	t.Run("WithFiles", func(t *testing.T) {
+		require.NoError(t, dir.WriteFile("1.sql", []byte("create table foo (id int)")))
+		require.NoError(t, dir.WriteFile("2.sql", []byte("create table bar (id int)")))
+
+		cmd := migrateCmd()
+		cmd.AddCommand(migrateApplyCmd())
+		s, err := runCmd(
+			cmd, "apply",
+			"-c", "file://"+path,
+			"--env", "local",
+			"--url", u,
+			"--var", "cloud_url="+srv.URL,
+		)
+		require.NoError(t, err)
+		// Reporting does not affect the output.
+		require.True(t, strings.HasPrefix(s, "Migrating to version 2 (2 migrations in total):"))
+		require.True(t, strings.HasSuffix(s, "  -- 2 migrations \n  -- 2 sql statements\n"))
+		require.Equal(t, "", report.FromVersion, "from empty database")
+		require.Equal(t, "2", report.ToVersion)
+		require.Equal(t, "2", report.CurrentVersion)
+		require.Len(t, report.Files, 2)
+		for i, n := range []string{"1.sql", "2.sql"} {
+			require.Equal(t, n, report.Files[i].Name)
+			require.Equal(t, 1, report.Files[i].Applied)
+			require.Zero(t, report.Files[i].Skipped)
+			require.Nil(t, report.Files[i].Error)
+		}
+	})
+
+	t.Run("PrintError", func(t *testing.T) {
+		status = http.StatusInternalServerError
+		require.NoError(t, dir.WriteFile("3.sql", []byte("create table baz (id int)")))
+		cmd := migrateCmd()
+		cmd.AddCommand(migrateApplyCmd())
+		s, err := runCmd(
+			cmd, "apply",
+			"-c", "file://"+path,
+			"--env", "local",
+			"--url", u,
+			"--var", "cloud_url="+srv.URL,
+		)
+		require.NoError(t, err)
+		// Reporting error should not affect the migration execution.
+		require.True(t, strings.HasSuffix(s, "  -- 1 migrations \n  -- 1 sql statements\nError: unexpected status code: 500\n"))
+
+		// Custom logging.
+		cmd = migrateCmd()
+		cmd.AddCommand(migrateApplyCmd())
+		s, err = runCmd(
+			cmd, "apply",
+			"-c", "file://"+path,
+			"--env", "local",
+			"--url", u,
+			"--var", "cloud_url="+srv.URL,
+			"--format", "{{ .Env.Driver }}",
+		)
+		require.NoError(t, err)
+		require.Equal(t, "sqlite3\nError: unexpected status code: 500\n", s)
+	})
+
+	t.Run("OpenError", func(t *testing.T) {
+		sqlclient.Register("openerror", sqlclient.OpenerFunc(func(ctx context.Context, u *url.URL) (*sqlclient.Client, error) {
+			return nil, errors.New("openerror")
+		}))
+		cmd := migrateCmd()
+		cmd.AddCommand(migrateApplyCmd())
+		_, err := runCmd(
+			cmd, "apply",
+			"-c", "file://"+path,
+			"--env", "local",
+			"--url", "openerror://db",
+			"--var", "cloud_url="+srv.URL,
+		)
+		require.EqualError(t, err, "openerror")
+		require.Equal(t, "Error: openerror", *reports.Error)
+	})
+}
+
+func TestMigrate_ApplyCloudReportSet(t *testing.T) {
+	var (
+		dir    migrate.MemDir
+		status int
+		report cloudapi.ReportMigrationSetInput
+		srv    = httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+			var m struct {
+				Query     string `json:"query"`
+				Variables struct {
+					Input json.RawMessage `json:"input"`
+				} `json:"variables"`
+			}
+			require.NoError(t, json.NewDecoder(r.Body).Decode(&m))
+			switch {
+			case strings.Contains(m.Query, "query"):
+				// Checksum before archiving.
+				hf, err := dir.Checksum()
+				require.NoError(t, err)
+				ht, err := hf.MarshalText()
+				require.NoError(t, err)
+				require.NoError(t, dir.WriteFile(migrate.HashFileName, ht))
+				// Archive and send.
+				arc, err := migrate.ArchiveDir(&dir)
+				require.NoError(t, err)
+				fmt.Fprintf(w, `{"data":{"dir":{"content":%q}}}`, base64.StdEncoding.EncodeToString(arc))
+			case strings.Contains(m.Query, "mutation"):
+				if status != 0 {
+					w.WriteHeader(status)
+				}
+				require.NoError(t, json.Unmarshal(m.Variables.Input, &report))
+			default:
+				t.Fatalf("unexpected query: %s", m.Query)
+			}
+		}))
+		h = `
+variable "cloud_url" {
+  type = string
+}
+
+atlas {
+  cloud {
+    token   = "token"
+    url     = var.cloud_url
+    project = "example"
+  }
+}
+
+data "remote_dir" "migration_set" {
+  name = "migration_set"
+}
+
+variable "urls" {
+  type    = list(string)
+  default = []
+}
+
+env {
+  name = atlas.env
+  for_each = toset(var.urls)
+  url = each.value
+  migration {
+    dir = data.remote_dir.migration_set.url
+  }
+}
+`
+		p    = t.TempDir()
+		path = filepath.Join(p, "atlas.hcl")
+	)
+	t.Cleanup(srv.Close)
+	require.NoError(t, os.WriteFile(path, []byte(h), 0600))
+
+	t.Run("MultipleEnvs", func(t *testing.T) {
+		cmd := migrateCmd()
+		cmd.AddCommand(migrateApplyCmd())
+		s, err := runCmd(
+			cmd, "apply",
+			"-c", "file://"+path,
+			"--env", "local",
+			"--var", fmt.Sprintf("urls=%s", openSQLite(t, "")),
+			"--var", fmt.Sprintf("urls=%s", openSQLite(t, "")),
+			"--var", "cloud_url="+srv.URL,
+		)
+		require.NoError(t, err)
+		require.Equal(t, "No migration files to execute\nNo migration files to execute\n", s)
+		require.NotEmpty(t, report.ID)
+		_, err = uuid.Parse(report.ID)
+		require.NoError(t, err, "set id is not a valid uuid")
+		require.False(t, report.StartTime.IsZero())
+		require.False(t, report.EndTime.IsZero())
+		require.Equal(t, 2, report.Planned)
+		require.Len(t, report.Completed, 2)
+		require.Nil(t, report.Error)
+
+		// Verify summary log.
+		require.Len(t, report.Log, 3)
+		top := report.Log[0]
+		require.Equal(t, top.Text, "Start migration for 2 targets")
+		require.Contains(t, top.Log[0].Text, "sqlite://file")
+		require.Contains(t, top.Log[1].Text, "sqlite://file")
+		require.Equal(t, report.Log[1].Text, "Run migration: 1")
+		require.Contains(t, report.Log[1].Log[0].Text, "Target URL: sqlite://file")
+		require.Contains(t, report.Log[1].Log[1].Text, "Migration directory: mem://migration_set")
+		require.Equal(t, report.Log[2].Text, "Run migration: 2")
+		require.Contains(t, report.Log[2].Log[0].Text, "Target URL: sqlite://file")
+		require.Contains(t, report.Log[2].Log[1].Text, "Migration directory: mem://migration_set")
+	})
+
+	t.Run("Error", func(t *testing.T) {
+		require.NoError(t, dir.WriteFile("1.sql", []byte("create table foo (id int)")))
+		require.NoError(t, dir.WriteFile("2.sql", []byte("create table bar (id int)")))
+
+		cmd := migrateCmd()
+		cmd.AddCommand(migrateApplyCmd())
+		s, err := runCmd(
+			cmd, "apply",
+			"-c", "file://"+path,
+			"--env", "local",
+			"--var", fmt.Sprintf("urls=%s", openSQLite(t, "create table bar (id int)")),
+			"--var", fmt.Sprintf("urls=%s", openSQLite(t, "create table bar (id int)")),
+			"--var", "cloud_url="+srv.URL,
+			"--allow-dirty",
+		)
+		require.EqualError(t, err, `sql/migrate: execute: executing statement "create table bar (id int)" from version "2": table bar already exists`)
+		require.Contains(t, s, "Migrating to version 2 (2 migrations in total):")
+		require.Contains(t, s, "Error: sql/migrate:")
+		require.NotEmpty(t, report.ID)
+		_, err = uuid.Parse(report.ID)
+		require.NoError(t, err, "set id is not a valid uuid")
+		require.False(t, report.StartTime.IsZero())
+		require.False(t, report.EndTime.IsZero())
+		require.Equal(t, 2, report.Planned)
+		require.Len(t, report.Completed, 1)
+		require.NotNil(t, report.Error)
+		require.Equal(t, `Error: sql/migrate: execute: executing statement "create table bar (id int)" from version "2": table bar already exists`, *report.Error)
+
+		// Verify summary log.
+		require.Len(t, report.Log, 2)
+		top := report.Log[0]
+		require.Equal(t, top.Text, "Start migration for 2 targets")
+		require.Contains(t, top.Log[0].Text, "sqlite://file")
+		require.Contains(t, top.Log[1].Text, "sqlite://file")
+		require.Equal(t, report.Log[1].Text, "Run migration: 1")
+		require.True(t, report.Log[1].Error)
+		require.Contains(t, report.Log[1].Log[0].Text, "Target URL: sqlite://file")
+		require.Contains(t, report.Log[1].Log[1].Text, "Migration directory: mem://migration_set")
+		require.Equal(t, `Error: sql/migrate: execute: executing statement "create table bar (id int)" from version "2": table bar already exists`, report.Log[1].Log[2].Text)
+	})
+}
+
 func TestMigrate_Diff(t *testing.T) {
 	p := t.TempDir()
 	to := hclURL(t)
@@ -921,6 +1266,93 @@ func TestMigrate_Diff(t *testing.T) {
 		require.Len(t, files, 1)
 		require.Equal(t, "CREATE TABLE `t` (`c` int NULL)", string(files[0].Bytes()))
 	})
+
+	t.Run("ProjectFile", func(t *testing.T) {
+		p := t.TempDir()
+		h := `
+variable "schema" {
+  type = string
+}
+
+variable "dir" {
+  type = string
+}
+
+variable "destructive" {
+  type = bool
+  default = false
+}
+
+env "local" {
+  src = "file://${var.schema}"
+  dev = "sqlite://ci?mode=memory&_fk=1"
+  migration {
+    dir = "file://${var.dir}"
+  }
+  diff {
+    skip {
+      drop_column = !var.destructive
+    }
+  }
+}
+`
+		pathC := filepath.Join(p, "atlas.hcl")
+		require.NoError(t, os.WriteFile(pathC, []byte(h), 0600))
+		pathS := filepath.Join(p, "schema.sql")
+		require.NoError(t, os.WriteFile(pathS, []byte(`CREATE TABLE t(c1 int, c2 int);`), 0600))
+		pathD := t.TempDir()
+		cmd := migrateCmd()
+		cmd.AddCommand(migrateDiffCmd())
+		s, err := runCmd(
+			cmd, "diff", "initial",
+			"-c", "file://"+pathC,
+			"--env", "local",
+			"--var", "schema="+pathS,
+			"--var", "dir="+pathD,
+		)
+		require.NoError(t, err)
+		require.Empty(t, s)
+		d, err := migrate.NewLocalDir(pathD)
+		require.NoError(t, err)
+		files, err := d.Files()
+		require.NoError(t, err)
+		require.Len(t, files, 1)
+		require.Equal(t, "-- Create \"t\" table\nCREATE TABLE `t` (`c1` int NULL, `c2` int NULL);\n", string(files[0].Bytes()))
+
+		// Drop column should be skipped.
+		require.NoError(t, os.WriteFile(pathS, []byte(`CREATE TABLE t(c1 int);`), 0600))
+		cmd = migrateCmd()
+		cmd.AddCommand(migrateDiffCmd())
+		s, err = runCmd(
+			cmd, "diff", "no_change",
+			"-c", "file://"+pathC,
+			"--env", "local",
+			"--var", "schema="+pathS,
+			"--var", "dir="+pathD,
+		)
+		require.NoError(t, err)
+		require.Equal(t, "The migration directory is synced with the desired state, no changes to be made\n", s)
+		files, err = d.Files()
+		require.NoError(t, err)
+		require.Len(t, files, 1)
+
+		// Column is dropped when destructive is true.
+		cmd = migrateCmd()
+		cmd.AddCommand(migrateDiffCmd())
+		s, err = runCmd(
+			cmd, "diff", "second",
+			"-c", "file://"+pathC,
+			"--env", "local",
+			"--var", "schema="+pathS,
+			"--var", "dir="+pathD,
+			"--var", "destructive=true",
+		)
+		require.NoError(t, err)
+		require.Empty(t, s)
+		files, err = d.Files()
+		require.NoError(t, err)
+		require.Len(t, files, 2)
+	})
 }
 
 func TestMigrate_StatusJSON(t *testing.T) {
@@ -1201,6 +1633,46 @@ func TestMigrate_Lint(t *testing.T) {
 	require.Error(t, err)
 	require.Equal(t, "2.sql", s)
 
+	t.Run("FromConfig", func(t *testing.T) {
+		cfg := filepath.Join(p, "atlas.hcl")
+		err := os.WriteFile(cfg, []byte(`
+variable "error" {
+  type    = bool
+  default = false
+}
+
+lint {
+  latest = 1
+  destructive {
+    error = var.error
+  }
+}
+`), 0600)
+		require.NoError(t, err)
+		cmd := migrateCmd()
+		cmd.AddCommand(migrateLintCmd())
+		s, err := runCmd(
+			cmd, "lint",
+			"--dir", "file://"+p,
+			"--dev-url", openSQLite(t, ""),
+			"-c", "file://"+cfg,
+		)
+		require.NoError(t, err)
+		require.Equal(t, "2.sql: destructive changes detected:\n\n\tL1: Dropping table \"t\"\n\n", s)
+
+		cmd = migrateCmd()
+		cmd.AddCommand(migrateLintCmd())
+		s, err = runCmd(
+			cmd, "lint",
+			"--dir", "file://"+p,
+			"--dev-url", openSQLite(t, ""),
+			"-c", "file://"+cfg,
+			"--var", "error=true",
+		)
+		require.Error(t, err)
+		require.Equal(t, "2.sql: destructive changes detected:\n\n\tL1: Dropping table \"t\"\n\n", s)
+	})
+
 	// Change files to golang-migrate format.
 	require.NoError(t, os.Rename(filepath.Join(p, "1.sql"), filepath.Join(p, "1.up.sql")))
 	require.NoError(t, os.Rename(filepath.Join(p, "2.sql"), filepath.Join(p, "1.down.sql")))
