@@ -255,6 +255,63 @@ table "users" {
 		require.NoError(t, err)
 		require.Equal(t, "-- Create \"users\" table\nCREATE TABLE `users` (\n\t`id` int NOT NULL\n);\n", s)
 	})
+
+	t.Run("SkipChanges", func(t *testing.T) {
+		var (
+			p   = t.TempDir()
+			cfg = filepath.Join(p, "atlas.hcl")
+		)
+		err = os.WriteFile(cfg, []byte(`
+variable "destructive" {
+  type = bool
+  default = false
+}
+
+env "local" {
+  diff {
+    skip {
+      drop_table = !var.destructive
+    }
+  }
+}
+`), 0600)
+		require.NoError(t, err)
+
+		// Skip destructive changes.
+		cmd := schemaCmd()
+		cmd.AddCommand(schemaDiffCmd())
+		s, err := runCmd(
+			cmd, "diff",
+			"-c", "file://"+cfg,
+			"--from", openSQLite(t, "create table users (id int);"),
+			"--to", openSQLite(t, ""),
+			"--env", "local",
+		)
+		require.NoError(t, err)
+		require.Equal(t, "Schemas are synced, no changes to be made.\n", s)
+
+		// Apply destructive changes.
+		cmd = schemaCmd()
+		cmd.AddCommand(schemaDiffCmd())
+		s, err = runCmd(
+			cmd, "diff",
+			"-c", "file://"+cfg,
+			"--from", openSQLite(t, "create table users (id int);"),
+			"--to", openSQLite(t, ""),
+			"--env", "local",
+			"--var", "destructive=true",
+		)
+		require.NoError(t, err)
+		lines := strings.Split(strings.TrimSpace(s), "\n")
+		require.Equal(t, []string{
+			"-- Disable the enforcement of foreign-keys constraints",
+			"PRAGMA foreign_keys = off;",
+			`-- Drop "users" table`,
+			"DROP TABLE `users`;",
+			"-- Enable back the enforcement of foreign-keys constraints",
+			"PRAGMA foreign_keys = on;",
+		}, lines)
+	})
 }
 
 func TestSchema_Apply(t *testing.T) {
@@ -441,6 +498,132 @@ func TestSchema_ApplyLog(t *testing.T) {
 	})
 }
 
+func TestSchema_ApplySchemaMismatch(t *testing.T) {
+	var (
+		p   = t.TempDir()
+		src = filepath.Join(p, "schema.hcl")
+	)
+	// SQLite always has a schema called "main".
+	err := os.WriteFile(src, []byte(`
+schema "hello" {}
+`), 0600)
+	require.NoError(t, err)
+	cmd := schemaCmd()
+	cmd.AddCommand(schemaApplyCmd())
+	_, err = runCmd(
+		cmd, "apply",
+		"-u", openSQLite(t, ""),
+		"-f", src,
+	)
+	require.EqualError(t, err, `mismatched HCL and database schemas: "main" <> "hello"`)
+}
+
+func TestSchema_ApplySkip(t *testing.T) {
+	var (
+		p   = t.TempDir()
+		cfg = filepath.Join(p, "atlas.hcl")
+		src = filepath.Join(p, "schema.hcl")
+	)
+	err := os.WriteFile(src, []byte(`
+schema "main" {}
+
+table "users" {
+  schema = schema.main
+  column "id" {
+    type = int
+  }
+}
+`), 0600)
+	require.NoError(t, err)
+	err = os.WriteFile(cfg, []byte(`
+variable "schema" {
+  type = string
+  default = "dev"
+}
+
+variable "destructive" {
+  type = bool
+  default = false
+}
+
+env "local" {
+  src = var.schema
+  dev_url = "sqlite://dev?mode=memory&_fk=1"
+}
+
+diff {
+  skip {
+    drop_table = !var.destructive
+  }
+}
+`), 0600)
+	require.NoError(t, err)
+
+	// Skip destructive changes.
+	cmd := schemaCmd()
+	cmd.AddCommand(schemaApplyCmd())
+	s, err := runCmd(
+		cmd, "apply",
+		"-u", openSQLite(t, "create table pets (id int);"),
+		"-c", "file://"+cfg,
+		"--var", "schema=file://"+src,
+		"--env", "local",
+		"--auto-approve",
+	)
+	require.NoError(t, err)
+	lines := strings.Split(strings.TrimSpace(s), "\n")
+	require.Equal(t, []string{
+		"-- Planned Changes:",
+		`-- Create "users" table`,
+		"CREATE TABLE `users` (`id` int NOT NULL);",
+	}, lines)
+
+	// Skip destructive changes by using project-level policy (no --env was passed).
+	cmd = schemaCmd()
+	cmd.AddCommand(schemaApplyCmd())
+	s, err = runCmd(
+		cmd, "apply",
+		"-u", openSQLite(t, "create table pets (id int);"),
+		"-c", "file://"+cfg, // Using the project-level policy.
+		"--to", "file://"+src,
+		"--dev-url", "sqlite://dev?mode=memory&_fk=1",
+		"--auto-approve",
+	)
+	require.NoError(t, err)
+	lines = strings.Split(strings.TrimSpace(s), "\n")
+	require.Equal(t, []string{
+		"-- Planned Changes:",
+		`-- Create "users" table`,
+		"CREATE TABLE `users` (`id` int NOT NULL);",
+	}, lines)
+
+	// Apply destructive changes.
+	cmd = schemaCmd()
+	cmd.AddCommand(schemaApplyCmd())
+	s, err = runCmd(
+		cmd, "apply",
+		"-u", openSQLite(t, "create table pets (id int);"),
+		"-c", "file://"+cfg,
+		"--var", "schema=file://"+src,
+		"--var", "destructive=true",
+		"--env", "local",
+		"--auto-approve",
+	)
+	require.NoError(t, err)
+	lines = strings.Split(strings.TrimSpace(s), "\n")
+	require.Equal(t, []string{
+		"-- Planned Changes:",
+		"-- Disable the enforcement of foreign-keys constraints",
+		"PRAGMA foreign_keys = off;",
+		`-- Drop "pets" table`,
+		"DROP TABLE `pets`;",
+		`-- Create "users" table`,
+		"CREATE TABLE `users` (`id` int NOT NULL);",
+		"-- Enable back the enforcement of foreign-keys constraints",
+		"PRAGMA foreign_keys = on;",
+	}, lines)
+}
+
 func TestSchema_ApplySources(t *testing.T) {
 	var (
 		p   = t.TempDir()
@@ -516,7 +699,7 @@ func TestSchema_ApplySQL(t *testing.T) {
 			"--auto-approve",
 		)
 		require.NoError(t, err)
-		require.Equal(t, "Schema is synced, no changes to be made.\n", s)
+		require.Equal(t, "Schema is synced, no changes to be made\n", s)
 	})
 	t.Run("Dir", func(t *testing.T) {
 		db := openSQLite(t, "")
@@ -538,7 +721,7 @@ func TestSchema_ApplySQL(t *testing.T) {
 			"--auto-approve",
 		)
 		require.NoError(t, err)
-		require.Equal(t, "Schema is synced, no changes to be made.\n", s)
+		require.Equal(t, "Schema is synced, no changes to be made\n", s)
 	})
 	t.Run("Error", func(t *testing.T) {
 		_, err := runCmd(
