@@ -50,7 +50,6 @@ var DataSources = []schemahcl.Option{
 	schemahcl.WithDataSource("runtimevar", RuntimeVarSrc),
 	schemahcl.WithDataSource("template_dir", TemplateDir),
 	schemahcl.WithDataSource("remote_dir", RemoteDir),
-	schemahcl.WithInitBlock("atlas", AtlasConfig),
 }
 
 // RuntimeVarSrc exposes the gocloud.dev/runtimevar as a schemahcl datasource.
@@ -310,43 +309,61 @@ func TemplateDir(ctx *hcl.EvalContext, block *hclsyntax.Block) (cty.Value, error
 	}), nil
 }
 
-// AtlasConfig is the handler for the "atlas" init block.
+// AtlasConfig exposes non-sensitive information returned by the "atlas" init-block.
+// By invoking AtlasInitBlock() a new config is returned that is set by the init block
+// defined and executed on schemahcl Eval functions.
+type AtlasConfig struct {
+	Client  *cloudapi.Client // Client attached to Atlas Cloud.
+	Project string           // Optional project.
+}
+
+// DefaultProjectName is the default name for projects.
+const DefaultProjectName = "default"
+
+// InitBlock returns the handler for the "atlas" init block.
 //
 //	atlas {
 //	  cloud {
-//	    token = data.runtimevar.token	// User token.
-//	    url   = var.cloud_url			// Optional URL.
+//	    token   = data.runtimevar.token  // User token.
+//	    url     = var.cloud_url          // Optional URL.
+//	    project = var.project            // Optional project. If set, cloud reporting is enabled.
 //	  }
 //	}
-func AtlasConfig(ctx *hcl.EvalContext, block *hclsyntax.Block) (cty.Value, error) {
-	var args struct {
-		Cloud struct {
-			Token string `hcl:"token"`
-			URL   string `hcl:"url,optional"`
-		} `hcl:"cloud,block"`
-	}
-	if diags := gohcl.DecodeBody(block.Body, ctx, &args); diags.HasErrors() {
-		return cty.NilVal, fmt.Errorf("atlas.cloud: decoding body: %v", diags)
-	}
-	cloud := cty.ObjectVal(map[string]cty.Value{
-		"client": cty.CapsuleVal(
-			clientType,
-			cloudapi.New(args.Cloud.URL, args.Cloud.Token),
-		),
+func (c *AtlasConfig) InitBlock() schemahcl.Option {
+	return schemahcl.WithInitBlock("atlas", func(ctx *hcl.EvalContext, block *hclsyntax.Block) (cty.Value, error) {
+		var args struct {
+			Cloud struct {
+				Token   string `hcl:"token"`
+				URL     string `hcl:"url,optional"`
+				Project string `hcl:"project,optional"`
+			} `hcl:"cloud,block"`
+		}
+		if diags := gohcl.DecodeBody(block.Body, ctx, &args); diags.HasErrors() {
+			return cty.NilVal, fmt.Errorf("atlas.cloud: decoding body: %v", diags)
+		}
+		if args.Cloud.Project == "" {
+			args.Cloud.Project = DefaultProjectName
+		}
+		c.Project = args.Cloud.Project
+		c.Client = cloudapi.New(args.Cloud.URL, args.Cloud.Token)
+		cloud := cty.ObjectVal(map[string]cty.Value{
+			"client":  cty.CapsuleVal(clientType, c.Client),
+			"project": cty.StringVal(args.Cloud.Project),
+		})
+		av, diags := (&hclsyntax.ScopeTraversalExpr{
+			Traversal: hcl.Traversal{hcl.TraverseRoot{Name: "atlas", SrcRange: block.Range()}},
+		}).Value(ctx)
+		switch {
+		case !diags.HasErrors():
+			m := av.AsValueMap()
+			m["cloud"] = cloud
+			return cty.ObjectVal(m), nil
+		case len(diags) == 1 && diags[0].Summary == "Unknown variable":
+			return cty.ObjectVal(map[string]cty.Value{"cloud": cloud}), nil
+		default:
+			return cty.NilVal, fmt.Errorf("atlas.cloud: getting config: %v", diags)
+		}
 	})
-	av, diags := (&hclsyntax.ScopeTraversalExpr{
-		Traversal: hcl.Traversal{hcl.TraverseRoot{Name: "atlas", SrcRange: block.Range()}},
-	}).Value(ctx)
-	switch {
-	case !diags.HasErrors():
-		m := av.AsValueMap()
-		m["cloud"] = cloud
-		return cty.ObjectVal(m), nil
-	case len(diags) == 1 && diags[0].Summary == "Unknown variable":
-		return cty.ObjectVal(map[string]cty.Value{"cloud": cloud}), nil
-	default:
-		return cty.NilVal, fmt.Errorf("atlas.cloud: getting config: %v", diags)
-	}
 }
 
 var clientType = cty.Capsule("client", reflect.TypeOf(cloudapi.Client{}))
@@ -435,11 +452,12 @@ type (
 
 	// MigrateDiffOptions for external migration differ.
 	MigrateDiffOptions struct {
-		Name   string
-		Indent string
-		To     []string
-		Dir    migrate.Dir
-		Dev    *sqlclient.Client
+		Name    string
+		Indent  string
+		To      []string
+		Dir     migrate.Dir
+		Dev     *sqlclient.Client
+		Options []schema.DiffOption
 	}
 	// MigrateDiffer allows external sources to implement custom migration differs.
 	MigrateDiffer interface {
@@ -448,10 +466,13 @@ type (
 	}
 )
 
-// States is a global registry for external state loaders.
-var States = registry{
-	"ent": EntLoader{},
-}
+var (
+	// States is a global registry for external state loaders.
+	States = registry{
+		"ent": EntLoader{},
+	}
+	errNotSchemaURL = errors.New("missing schema in --dev-url. See: https://atlasgo.io/url")
+)
 
 type registry map[string]StateLoader
 
@@ -477,8 +498,12 @@ type EntLoader struct{}
 // LoadState returns a migrate.StateReader that reads the schema from an ent.Schema.
 func (l EntLoader) LoadState(ctx context.Context, opts *LoadStateOptions) (migrate.StateReader, error) {
 	switch {
-	case opts.Dev == nil, len(opts.URLs) != 1:
-		return nil, errors.New("schema url and dev database are required")
+	case len(opts.URLs) != 1:
+		return nil, errors.New(`"ent://" requires exactly one schema URL`)
+	case opts.Dev == nil:
+		return nil, errors.New(`required flag "--dev-url" not set`)
+	case opts.Dev.URL.Schema == "":
+		return nil, errNotSchemaURL
 	case opts.URLs[0].Query().Has("globalid"):
 		return nil, errors.New("globalid is not supported by this command. Use 'migrate diff' instead")
 	}
@@ -499,6 +524,16 @@ func (l EntLoader) LoadState(ctx context.Context, opts *LoadStateOptions) (migra
 			return nil, err
 		}
 	}
+	if len(realm.Schemas) != 1 {
+		return nil, fmt.Errorf("expect exactly one schema, got %d", len(realm.Schemas))
+	}
+	// Use the dev-database schema name if the schema name is empty.
+	if realm.Schemas[0].Name == "" && opts.Dev.URL.Schema != "" {
+		realm.Schemas[0].Name = opts.Dev.URL.Schema
+	}
+	for _, t := range realm.Schemas[0].Tables {
+		t.Schema = realm.Schemas[0]
+	}
 	return migrate.Realm(realm), nil
 }
 
@@ -507,6 +542,9 @@ func (l EntLoader) MigrateDiff(ctx context.Context, opts *MigrateDiffOptions) er
 	if !l.needDiff(opts.To) {
 		return errors.New("invalid diff call")
 	}
+	if opts.Dev.URL.Schema == "" {
+		return errNotSchemaURL
+	}
 	u, err := url.Parse(opts.To[0])
 	if err != nil {
 		return nil
@@ -525,6 +563,7 @@ func (l EntLoader) MigrateDiff(ctx context.Context, opts *MigrateDiffOptions) er
 		entschema.WithGlobalUniqueID(true),
 		entschema.WithIndent(opts.Indent),
 		entschema.WithMigrationMode(entschema.ModeReplay),
+		entschema.WithDiffOptions(opts.Options...),
 	)
 	if err != nil {
 		return fmt.Errorf("creating migrate reader: %w", err)
