@@ -16,9 +16,6 @@ import (
 	"ariga.io/atlas/sql/migrate"
 	"ariga.io/atlas/sql/postgres"
 	"ariga.io/atlas/sql/schema"
-	"entgo.io/ent/dialect"
-	entschema "entgo.io/ent/dialect/sql/schema"
-	entmigrate "entgo.io/ent/entc/integration/ent/migrate"
 
 	_ "github.com/lib/pq"
 	"github.com/stretchr/testify/require"
@@ -272,22 +269,6 @@ func TestCockroach_Enums(t *testing.T) {
 	})
 }
 
-func TestCockroach_Ent(t *testing.T) {
-	crdbRun(t, func(t *crdbTest) {
-		// Cockroach doesn't support macaddr but its in the integration tests of ent
-		for _, ff := range entmigrate.FieldTypesColumns {
-			if st := ff.SchemaType; st[dialect.Postgres] == "macaddr" {
-				c := ff
-				t.Cleanup(func() {
-					c.SchemaType = st
-				})
-				c.SchemaType = nil
-			}
-		}
-		testEntIntegration(t, dialect.Postgres, t.db, entschema.WithAtlas(true))
-	})
-}
-
 func TestCockroach_HCL(t *testing.T) {
 	full := `
 schema "public" {
@@ -369,17 +350,17 @@ func TestCockroach_CLI(t *testing.T) {
 			}`
 	t.Run("SchemaInspect", func(t *testing.T) {
 		crdbRun(t, func(t *crdbTest) {
-			testCLISchemaInspect(t, h, t.dsn(), postgres.EvalHCL, "-s", "public")
+			testCLISchemaInspect(t, h, t.url(""), postgres.EvalHCL, "-s", "public")
 		})
 	})
 	t.Run("SchemaApply", func(t *testing.T) {
 		crdbRun(t, func(t *crdbTest) {
-			testCLISchemaApply(t, h, t.dsn(), "-s", "public")
+			testCLISchemaApply(t, h, t.url(""), "-s", "public")
 		})
 	})
 	t.Run("SchemaApplyDryRun", func(t *testing.T) {
 		crdbRun(t, func(t *crdbTest) {
-			testCLISchemaApplyDry(t, h, t.dsn())
+			testCLISchemaApplyDry(t, h, t.url(""))
 		})
 	})
 	t.Run("SchemaApplyWithVars", func(t *testing.T) {
@@ -398,17 +379,17 @@ table "users" {
 }
 `
 		crdbRun(t, func(t *crdbTest) {
-			testCLISchemaApply(t, h, t.dsn(), "--var", "tenant=public", "-s", "public")
+			testCLISchemaApply(t, h, t.url(""), "--var", "tenant=public", "-s", "public")
 		})
 	})
 	t.Run("SchemaDiffRun", func(t *testing.T) {
 		crdbRun(t, func(t *crdbTest) {
-			testCLISchemaDiff(t, t.dsn())
+			testCLISchemaDiff(t, t.url(""))
 		})
 	})
 	t.Run("SchemaApplyAutoApprove", func(t *testing.T) {
 		crdbRun(t, func(t *crdbTest) {
-			testCLISchemaApplyAutoApprove(t, h, t.dsn(), "-s", "public")
+			testCLISchemaApplyAutoApprove(t, h, t.url(""), "-s", "public")
 		})
 	})
 }
@@ -441,14 +422,14 @@ func TestCockroach_CLI_MultiSchema(t *testing.T) {
 		crdbRun(t, func(t *crdbTest) {
 			t.dropSchemas("test2")
 			t.dropTables("users")
-			testCLIMultiSchemaInspect(t, h, t.dsn(), []string{"public", "test2"}, postgres.EvalHCL)
+			testCLIMultiSchemaInspect(t, h, t.url(""), []string{"public", "test2"}, postgres.EvalHCL)
 		})
 	})
 	t.Run("SchemaApply", func(t *testing.T) {
 		crdbRun(t, func(t *crdbTest) {
 			t.dropSchemas("test2")
 			t.dropTables("users")
-			testCLIMultiSchemaApply(t, h, t.dsn(), []string{"public", "test2"}, postgres.EvalHCL)
+			testCLIMultiSchemaApply(t, h, t.url(""), []string{"public", "test2"}, postgres.EvalHCL)
 		})
 	})
 }
@@ -806,11 +787,7 @@ create table atlas_types_sanity
 	})
 }
 
-func (t *crdbTest) url() string {
-	return t.dsn()
-}
-
-func (t *crdbTest) dsn() string {
+func (t *crdbTest) url(_ string) string {
 	return fmt.Sprintf("postgres://root:pass@localhost:%d/defaultdb?sslmode=disable", t.port)
 }
 
@@ -924,24 +901,6 @@ func (t *crdbTest) posts() *schema.Table {
 	return postsT
 }
 
-func (t *crdbTest) revisions() *schema.Table {
-	versionsT := &schema.Table{
-		Name: "atlas_schema_revisions",
-		Columns: []*schema.Column{
-			{Name: "version", Type: &schema.ColumnType{Type: &schema.StringType{T: "character varying"}}},
-			{Name: "description", Type: &schema.ColumnType{Type: &schema.StringType{T: "character varying"}}},
-			{Name: "execution_state", Type: &schema.ColumnType{Type: &schema.StringType{T: "character varying"}}},
-			{Name: "executed_at", Type: &schema.ColumnType{Type: &schema.TimeType{T: "timestamp with time zone"}}},
-			{Name: "execution_time", Type: &schema.ColumnType{Type: &schema.IntegerType{T: "bigint"}}},
-			{Name: "hash", Type: &schema.ColumnType{Type: &schema.StringType{T: "character varying"}}},
-			{Name: "operator_version", Type: &schema.ColumnType{Type: &schema.StringType{T: "character varying"}}},
-			{Name: "meta", Type: &schema.ColumnType{Type: &schema.JSONType{T: "jsonb"}, Raw: "jsonb"}},
-		},
-	}
-	versionsT.PrimaryKey = &schema.Index{Parts: []*schema.IndexPart{{C: versionsT.Columns[0]}}}
-	return versionsT
-}
-
 func (t *crdbTest) realm() *schema.Realm {
 	r := &schema.Realm{
 		Schemas: []*schema.Schema{
