@@ -18,7 +18,6 @@ import (
 	"ariga.io/atlas/sql/migrate"
 	"ariga.io/atlas/sql/mysql"
 	"ariga.io/atlas/sql/schema"
-	"entgo.io/ent/dialect"
 	_ "github.com/go-sql-driver/mysql"
 	"github.com/stretchr/testify/require"
 )
@@ -495,12 +494,6 @@ func TestMySQL_ForeignKey(t *testing.T) {
 	})
 }
 
-func TestMySQL_Ent(t *testing.T) {
-	myRun(t, func(t *myTest) {
-		testEntIntegration(t, dialect.MySQL, t.db)
-	})
-}
-
 func TestMySQL_AdvisoryLock(t *testing.T) {
 	myRun(t, func(t *myTest) {
 		testAdvisoryLock(t.T, t.drv.(schema.Locker))
@@ -924,6 +917,7 @@ create table atlas_types_sanity
 				Attrs: []schema.Attr{
 					&schema.Charset{V: "latin1"},
 					&schema.Collation{V: "latin1_swedish_ci"},
+					&mysql.Engine{V: "InnoDB", Default: true},
 				},
 				Schema: realm.Schemas[0],
 				Columns: []*schema.Column{
@@ -1274,12 +1268,14 @@ create table atlas_types_sanity
 						return []schema.Attr{
 							&schema.Charset{V: "latin1"},
 							&schema.Collation{V: "latin1_swedish_ci"},
+							&mysql.Engine{V: "InnoDB", Default: true},
 							&schema.Check{Name: "tJSON", Expr: "json_valid(`tJSON`)"},
 						}
 					}
 					return []schema.Attr{
 						&schema.Charset{V: "latin1"},
 						&schema.Collation{V: "latin1_swedish_ci"},
+						&mysql.Engine{V: "InnoDB", Default: true},
 					}
 				}(),
 				Schema: realm.Schemas[0],
