@@ -20,7 +20,6 @@ import (
 	"ariga.io/atlas/sql/postgres"
 	"ariga.io/atlas/sql/schema"
 	"ariga.io/atlas/sql/sqlclient"
-	"entgo.io/ent/dialect"
 	_ "github.com/lib/pq"
 	"github.com/stretchr/testify/require"
 )
@@ -36,12 +35,13 @@ type pgTest struct {
 }
 
 var pgTests = map[string]*pgTest{
-	"postgres10": {port: 5430},
-	"postgres11": {port: 5431},
-	"postgres12": {port: 5432},
-	"postgres13": {port: 5433},
-	"postgres14": {port: 5434},
-	"postgres15": {port: 5435},
+	"postgres-ext-postgis": {port: 5429},
+	"postgres10":           {port: 5430},
+	"postgres11":           {port: 5431},
+	"postgres12":           {port: 5432},
+	"postgres13":           {port: 5433},
+	"postgres14":           {port: 5434},
+	"postgres15":           {port: 5435},
 }
 
 func pgRun(t *testing.T, fn func(*pgTest)) {
@@ -57,6 +57,21 @@ func pgRun(t *testing.T, fn func(*pgTest)) {
 						log.Fatalln(err)
 					}
 					dbs = append(dbs, tt.db) // close connection after all tests have been run
+					// the postgis/postgis image enables the postgis_topology and postgis_tiger_geocoder extensions,
+					// this creates a few unwanted schemas, so we drop it.
+					// https://github.com/postgis/docker-postgis/issues/187
+					if tt.version == "postgres-ext-postgis" {
+						schemasToDrop := []string{
+							"tiger",      // created by postgis_tiger_geocoder
+							"tiger_data", // created by postgis_tiger_geocoder
+							"topology",   // created by postgis_topology
+						}
+						for _, s := range schemasToDrop {
+							if _, err := tt.db.Exec("DROP SCHEMA IF EXISTS " + s + " CASCADE;"); err != nil {
+								log.Fatalf("error dropping schema %q: %v", s, err)
+							}
+						}
+					}
 					tt.drv, err = postgres.Open(tt.db)
 					if err != nil {
 						log.Fatalln(err)
@@ -414,54 +429,49 @@ func TestPostgres_ForeignKey(t *testing.T) {
 	})
 }
 
-func TestPostgres_Ent(t *testing.T) {
-	pgRun(t, func(t *pgTest) {
-		testEntIntegration(t, dialect.Postgres, t.db)
-	})
+func TestPostgres_EntGlobalUniqueID(t *testing.T) {
 	// Migration to global unique identifiers.
-	t.Run("GlobalUniqueID", func(t *testing.T) {
-		pgRun(t, func(t *pgTest) {
-			ctx := context.Background()
-			t.dropTables("global_id")
-			_, err := t.driver().ExecContext(ctx, "CREATE TABLE global_id (id int NOT NULL GENERATED BY DEFAULT AS IDENTITY, PRIMARY KEY(id))")
-			require.NoError(t, err)
-			_, err = t.driver().ExecContext(ctx, "ALTER TABLE global_id ALTER COLUMN id RESTART WITH 1024")
-			require.NoError(t, err)
-			_, err = t.driver().ExecContext(ctx, "INSERT INTO global_id VALUES (default), (default)")
-			require.NoError(t, err)
-			var id int
-			require.NoError(t, t.db.QueryRow("SELECT id FROM global_id").Scan(&id))
-			require.Equal(t, 1024, id)
-			_, err = t.driver().ExecContext(ctx, "DELETE FROM global_id WHERE id = 1024")
-			require.NoError(t, err)
+	pgRun(t, func(t *pgTest) {
+		ctx := context.Background()
+		t.dropTables("global_id")
+		_, err := t.driver().ExecContext(ctx, "CREATE TABLE global_id (id int NOT NULL GENERATED BY DEFAULT AS IDENTITY, PRIMARY KEY(id))")
+		require.NoError(t, err)
+		_, err = t.driver().ExecContext(ctx, "ALTER TABLE global_id ALTER COLUMN id RESTART WITH 1024")
+		require.NoError(t, err)
+		_, err = t.driver().ExecContext(ctx, "INSERT INTO global_id VALUES (default), (default)")
+		require.NoError(t, err)
+		var id int
+		require.NoError(t, t.db.QueryRow("SELECT id FROM global_id").Scan(&id))
+		require.Equal(t, 1024, id)
+		_, err = t.driver().ExecContext(ctx, "DELETE FROM global_id WHERE id = 1024")
+		require.NoError(t, err)
 
-			globalT := t.loadTable("global_id")
-			c, ok := globalT.Column("id")
-			require.True(t, ok)
-			require.EqualValues(t, 1, c.Attrs[0].(*postgres.Identity).Sequence.Start)
-			t.migrate(&schema.ModifyTable{
-				T: globalT,
-				Changes: []schema.Change{
-					&schema.ModifyColumn{
-						From: globalT.Columns[0],
-						To: schema.NewIntColumn("id", "int").
-							AddAttrs(&postgres.Identity{
-								Generation: "BY DEFAULT",
-								Sequence: &postgres.Sequence{
-									Start: 1024,
-								},
-							}),
-						Change: schema.ChangeAttr,
-					},
+		globalT := t.loadTable("global_id")
+		c, ok := globalT.Column("id")
+		require.True(t, ok)
+		require.EqualValues(t, 1, c.Attrs[0].(*postgres.Identity).Sequence.Start)
+		t.migrate(&schema.ModifyTable{
+			T: globalT,
+			Changes: []schema.Change{
+				&schema.ModifyColumn{
+					From: globalT.Columns[0],
+					To: schema.NewIntColumn("id", "int").
+						AddAttrs(&postgres.Identity{
+							Generation: "BY DEFAULT",
+							Sequence: &postgres.Sequence{
+								Start: 1024,
+							},
+						}),
+					Change: schema.ChangeAttr,
 				},
-			})
-			_, err = t.driver().ExecContext(ctx, "INSERT INTO global_id VALUES (default), (default)")
-			require.NoError(t, err)
-			globalT = t.loadTable("global_id")
-			c, ok = globalT.Column("id")
-			require.True(t, ok)
-			require.EqualValues(t, 1024, c.Attrs[0].(*postgres.Identity).Sequence.Start)
+			},
 		})
+		_, err = t.driver().ExecContext(ctx, "INSERT INTO global_id VALUES (default), (default)")
+		require.NoError(t, err)
+		globalT = t.loadTable("global_id")
+		c, ok = globalT.Column("id")
+		require.True(t, ok)
+		require.EqualValues(t, 1024, c.Attrs[0].(*postgres.Identity).Sequence.Start)
 	})
 }
 
