@@ -82,6 +82,7 @@ func TestSQLite_Script(t *testing.T) {
 			"apply":       tt.cmdApply,
 			"exist":       tt.cmdExist,
 			"synced":      tt.cmdSynced,
+			"cmphcl":      tt.cmdCmpHCL,
 			"cmpshow":     tt.cmdCmpShow,
 			"cmpmig":      tt.cmdCmpMig,
 			"execsql":     tt.cmdExec,
@@ -330,6 +331,18 @@ func (t *pgTest) cmdCmpHCL(ts *testscript.TestScript, _ bool, args []string) {
 	})
 }
 
+func (t *liteTest) cmdCmpHCL(ts *testscript.TestScript, _ bool, args []string) {
+	cmdCmpHCL(ts, args, func(name string) (string, error) {
+		s, err := ts.Value(keyDrv).(migrate.Driver).InspectSchema(context.Background(), "main", nil)
+		ts.Check(err)
+		buf, err := sqlite.MarshalHCL(s)
+		require.NoError(t, err)
+		return string(buf), nil
+	}, func(s string) string {
+		return strings.ReplaceAll(ts.ReadFile(s), "$db", ts.Getenv("db"))
+	})
+}
+
 func cmdCmpHCL(ts *testscript.TestScript, args []string, inspect func(schema string) (string, error), read ...func(string) string) {
 	if len(args) != 1 {
 		ts.Fatalf("invalid number of args to 'cmpinspect': %d", len(args))
