@@ -14,12 +14,6 @@ import (
 	"ariga.io/atlas/sql/mysql"
 	"ariga.io/atlas/sql/schema"
 
-	"entgo.io/ent/dialect"
-	entsql "entgo.io/ent/dialect/sql"
-	entschema "entgo.io/ent/dialect/sql/schema"
-	"entgo.io/ent/entc/integration/ent"
-	"entgo.io/ent/entc/integration/ent/migrate"
-
 	_ "github.com/go-sql-driver/mysql"
 	"github.com/stretchr/testify/require"
 )
@@ -538,7 +532,7 @@ func TestTiDB_CLI_MultiSchema(t *testing.T) {
 			t.dropTables("users")
 			attrs := t.defaultAttrs()
 			charset, collate := attrs[0].(*schema.Charset), attrs[1].(*schema.Collation)
-			testCLIMultiSchemaInspect(t, fmt.Sprintf(h, charset.V, collate.V, charset.V, collate.V), t.dsn(""), []string{"test", "test2"}, mysql.EvalHCL)
+			testCLIMultiSchemaInspect(t, fmt.Sprintf(h, charset.V, collate.V, charset.V, collate.V), t.url(""), []string{"test", "test2"}, mysql.EvalHCL)
 		})
 	})
 	t.Run("SchemaApply", func(t *testing.T) {
@@ -547,7 +541,7 @@ func TestTiDB_CLI_MultiSchema(t *testing.T) {
 			t.dropTables("users")
 			attrs := t.defaultAttrs()
 			charset, collate := attrs[0].(*schema.Charset), attrs[1].(*schema.Collation)
-			testCLIMultiSchemaApply(t, fmt.Sprintf(h, charset.V, collate.V, charset.V, collate.V), t.dsn(""), []string{"test", "test2"}, mysql.EvalHCL)
+			testCLIMultiSchemaApply(t, fmt.Sprintf(h, charset.V, collate.V, charset.V, collate.V), t.url(""), []string{"test", "test2"}, mysql.EvalHCL)
 		})
 	})
 }
@@ -571,14 +565,14 @@ func TestTiDB_CLI(t *testing.T) {
 		tidbRun(t, func(t *myTest) {
 			attrs := t.defaultAttrs()
 			charset, collate := attrs[0].(*schema.Charset), attrs[1].(*schema.Collation)
-			testCLISchemaInspect(t, fmt.Sprintf(h, charset.V, collate.V), t.dsn("test"), mysql.EvalHCL)
+			testCLISchemaInspect(t, fmt.Sprintf(h, charset.V, collate.V), t.url("test"), mysql.EvalHCL)
 		})
 	})
 	t.Run("SchemaApply", func(t *testing.T) {
 		tidbRun(t, func(t *myTest) {
 			attrs := t.defaultAttrs()
 			charset, collate := attrs[0].(*schema.Charset), attrs[1].(*schema.Collation)
-			testCLISchemaApply(t, fmt.Sprintf(h, charset.V, collate.V), t.dsn("test"))
+			testCLISchemaApply(t, fmt.Sprintf(h, charset.V, collate.V), t.url("test"))
 		})
 	})
 	t.Run("SchemaApplyWithVars", func(t *testing.T) {
@@ -597,19 +591,19 @@ table "users" {
 }
 `
 		tidbRun(t, func(t *myTest) {
-			testCLISchemaApply(t, h, t.dsn("test"), "--var", "tenant=test")
+			testCLISchemaApply(t, h, t.url("test"), "--var", "tenant=test")
 		})
 	})
 	t.Run("SchemaApplyDryRun", func(t *testing.T) {
 		tidbRun(t, func(t *myTest) {
 			attrs := t.defaultAttrs()
 			charset, collate := attrs[0].(*schema.Charset), attrs[1].(*schema.Collation)
-			testCLISchemaApplyDry(t, fmt.Sprintf(h, charset.V, collate.V), t.dsn("test"))
+			testCLISchemaApplyDry(t, fmt.Sprintf(h, charset.V, collate.V), t.url("test"))
 		})
 	})
 	t.Run("SchemaDiffRun", func(t *testing.T) {
 		tidbRun(t, func(t *myTest) {
-			testCLISchemaDiff(t, t.dsn("test"))
+			testCLISchemaDiff(t, t.url("test"))
 		})
 	})
 }
@@ -657,45 +651,6 @@ schema "test" {
 	})
 }
 
-func TestTiDB_Ent_EntEngine(t *testing.T) {
-	tidbRun(t, func(t *myTest) {
-		testEntIntegration(t, dialect.MySQL, t.db, migrate.WithForeignKeys(false))
-	})
-}
-
-func TestTiDB_Ent_AtlasEngine(t *testing.T) {
-	tidbRun(t, func(t *myTest) {
-		ctx := context.Background()
-		drv := entsql.OpenDB(dialect.MySQL, t.db)
-		client := ent.NewClient(ent.Driver(drv))
-		require.NoError(t, client.Schema.Create(ctx, entschema.WithAtlas(true)))
-		sanity(client)
-		realm := t.loadRealm()
-		ensureNoChange(t, realm.Schemas[0].Tables...)
-
-		// Drop tables.
-		changes := make([]schema.Change, len(realm.Schemas[0].Tables))
-		for i, t := range realm.Schemas[0].Tables {
-			changes[i] = &schema.DropTable{T: t}
-		}
-		t.migrate(changes...)
-
-		// Add tables.
-		for i, t := range realm.Schemas[0].Tables {
-			changes[i] = &schema.AddTable{T: t}
-		}
-		t.migrate(changes...)
-		ensureNoChange(t, realm.Schemas[0].Tables...)
-		sanity(client)
-
-		// Drop tables.
-		for i, t := range realm.Schemas[0].Tables {
-			changes[i] = &schema.DropTable{T: t}
-		}
-		t.migrate(changes...)
-	})
-}
-
 func TestTiDB_Sanity(t *testing.T) {
 	n := "atlas_types_sanity"
 	t.Run("Common", func(t *testing.T) {
@@ -749,9 +704,8 @@ create table atlas_types_sanity
 				Name: n,
 				Attrs: []schema.Attr{
 					&schema.Charset{V: "latin1"},
-					&schema.Collation{
-						V: "latin1_bin",
-					},
+					&schema.Collation{V: "latin1_bin"},
+					&mysql.Engine{V: "InnoDB", Default: true},
 				},
 				Schema: realm.Schemas[0],
 				Columns: []*schema.Column{
@@ -1016,6 +970,7 @@ create table atlas_types_sanity
 					return []schema.Attr{
 						&schema.Charset{V: "latin1"},
 						&schema.Collation{V: "latin1_bin"},
+						&mysql.Engine{V: "InnoDB", Default: true},
 					}
 				}(),
 				Schema: realm.Schemas[0],
@@ -1053,6 +1008,7 @@ create table atlas_types_sanity
 					return []schema.Attr{
 						&schema.Collation{V: "utf8mb4_bin"},
 						&schema.Charset{V: "utf8mb4"},
+						&mysql.Engine{V: "InnoDB", Default: true},
 					}
 				}(),
 				Columns: []*schema.Column{
