@@ -86,7 +86,7 @@ func (s *State) evalReferences(ctx *hcl.EvalContext, body *hclsyntax.Body) error
 	)
 	for _, b := range body.Blocks {
 		switch b := b; {
-		case b.Type == dataBlock:
+		case b.Type == BlockData:
 			if len(b.Labels) < 2 {
 				return fmt.Errorf("data block %q must have exactly 2 labels", b.Type)
 			}
@@ -96,18 +96,18 @@ func (s *State) evalReferences(ctx *hcl.EvalContext, body *hclsyntax.Body) error
 			}
 			// Data references are combined from
 			// "data", "source" and "name" labels.
-			addr := [3]string{dataBlock, b.Labels[0], b.Labels[1]}
+			addr := [3]string{RefData, b.Labels[0], b.Labels[1]}
 			nodes[addr] = &node{
 				addr:  addr,
 				value: func() (cty.Value, error) { return h(ctx, b) },
 				edges: func() []hcl.Traversal { return bodyVars(b.Body) },
 			}
-		case b.Type == localsBlock:
+		case b.Type == BlockLocals:
 			for k, v := range b.Body.Attributes {
 				k, v := k, v
 				// Local references are combined from
 				// "local" and "name" labels.
-				addr := [3]string{localRef, k, ""}
+				addr := [3]string{RefLocal, k, ""}
 				nodes[addr] = &node{
 					addr:  addr,
 					edges: func() []hcl.Traversal { return hclsyntax.Variables(v.Expr) },
@@ -160,10 +160,10 @@ func (s *State) evalReferences(ctx *hcl.EvalContext, body *hclsyntax.Body) error
 		for _, e := range n.edges() {
 			var addr [3]string
 			switch root := e.RootName(); {
-			case root == localRef && len(e) == 2:
-				addr = [3]string{localRef, e[1].(hcl.TraverseAttr).Name, ""}
-			case root == dataBlock && len(e) > 2:
-				addr = [3]string{dataBlock, e[1].(hcl.TraverseAttr).Name, e[2].(hcl.TraverseAttr).Name}
+			case root == RefLocal && len(e) == 2:
+				addr = [3]string{RefLocal, e[1].(hcl.TraverseAttr).Name, ""}
+			case root == RefData && len(e) > 2:
+				addr = [3]string{RefData, e[1].(hcl.TraverseAttr).Name, e[2].(hcl.TraverseAttr).Name}
 			case s.config.initblk[root] != nil && len(e) == 1:
 				addr = [3]string{root, "", ""}
 			}
@@ -181,9 +181,9 @@ func (s *State) evalReferences(ctx *hcl.EvalContext, body *hclsyntax.Body) error
 			return err
 		}
 		switch n.addr[0] {
-		case dataBlock:
+		case RefData:
 			data := make(map[string]cty.Value)
-			if vv, ok := ctx.Variables[dataBlock]; ok {
+			if vv, ok := ctx.Variables[RefData]; ok {
 				data = vv.AsValueMap()
 			}
 			src := make(map[string]cty.Value)
@@ -192,14 +192,14 @@ func (s *State) evalReferences(ctx *hcl.EvalContext, body *hclsyntax.Body) error
 			}
 			src[n.addr[2]] = v
 			data[n.addr[1]] = cty.ObjectVal(src)
-			ctx.Variables[dataBlock] = cty.ObjectVal(data)
-		case localRef:
+			ctx.Variables[RefData] = cty.ObjectVal(data)
+		case RefLocal:
 			locals := make(map[string]cty.Value)
-			if vv, ok := ctx.Variables[localRef]; ok {
+			if vv, ok := ctx.Variables[RefLocal]; ok {
 				locals = vv.AsValueMap()
 			}
 			locals[n.addr[1]] = v
-			ctx.Variables[localRef] = cty.ObjectVal(locals)
+			ctx.Variables[RefLocal] = cty.ObjectVal(locals)
 		default:
 			ctx.Variables[n.addr[0]] = v
 		}
@@ -221,15 +221,15 @@ func (s *State) evalReferences(ctx *hcl.EvalContext, body *hclsyntax.Body) error
 	return nil
 }
 
-func mergeCtxVar(ctx *hcl.EvalContext, vals map[string]cty.Value) {
-	v, ok := ctx.Variables[varRef]
+func mergeCtxVar(ctx *hcl.EvalContext, values map[string]cty.Value) {
+	v, ok := ctx.Variables[RefVar]
 	if ok {
 		v.ForEachElement(func(key cty.Value, val cty.Value) (stop bool) {
-			vals[key.AsString()] = val
+			values[key.AsString()] = val
 			return false
 		})
 	}
-	ctx.Variables[varRef] = cty.ObjectVal(vals)
+	ctx.Variables[RefVar] = cty.ObjectVal(values)
 }
 
 func setBlockVars(ctx *hcl.EvalContext, b *hclsyntax.Body) (*hcl.EvalContext, error) {
@@ -374,15 +374,60 @@ var (
 
 // Built-in blocks.
 const (
-	varBlock    = "variable"
-	dataBlock   = "data"
-	localsBlock = "locals"
-	forEachAttr = "for_each"
-	eachRef     = "each"
-	varRef      = "var"
-	localRef    = "local"
+	BlockData     = "data"
+	BlockLocals   = "locals"
+	BlockVariable = "variable"
+	RefData       = "data"
+	RefVar        = "var"
+	RefLocal      = "local"
+	AttrName      = "name"
+	forEachAttr   = "for_each"
+	eachRef       = "each"
 )
 
+// Variables represents the dynamic variables used in a body.
+type Variables struct {
+	Var, Local map[string]bool
+	Data       map[string]map[string]bool
+}
+
+// DynamicRefs returns the references to dynamic variables.
+// i.e, variables, locals, and data sources.
+func DynamicRefs(b *hclsyntax.Body) *Variables {
+	var (
+		vars = &Variables{
+			Var:   make(map[string]bool),
+			Local: make(map[string]bool),
+			Data:  make(map[string]map[string]bool),
+		}
+		visit func(b *hclsyntax.Body, vars *Variables)
+	)
+	visit = func(b *hclsyntax.Body, vars *Variables) {
+		for _, a := range b.Attributes {
+			for _, v := range hclsyntax.Variables(a.Expr) {
+				switch {
+				case len(v) == 0:
+				case v.RootName() == RefVar && len(v) > 1:
+					vars.Var[v[1].(hcl.TraverseAttr).Name] = true
+				case v.RootName() == RefLocal && len(v) > 1:
+					vars.Local[v[1].(hcl.TraverseAttr).Name] = true
+				case v.RootName() == RefData && len(v) > 2:
+					n1, n2 := v[1].(hcl.TraverseAttr).Name, v[2].(hcl.TraverseAttr).Name
+					if _, ok := vars.Data[n1]; !ok {
+						vars.Data[n1] = make(map[string]bool)
+					}
+					vars.Data[n1][n2] = true
+				}
+			}
+		}
+		for _, b := range b.Blocks {
+			visit(b.Body, vars)
+		}
+	}
+	visit(b, vars)
+	return vars
+}
+
 // defRegistry returns a tree of blockDef structs representing the schema of the
 // blocks in the *hclsyntax.Body. The returned fields and children of each type
 // are an intersection of all existing blocks of the same type.
@@ -393,7 +438,7 @@ func defRegistry(b *hclsyntax.Body) *blockDef {
 	}
 	for _, blk := range b.Blocks {
 		// variable definition blocks are available in the HCL source but not reachable by reference.
-		if blk.Type == varBlock {
+		if blk.Type == BlockVariable {
 			continue
 		}
 		reg.child(extractDef(blk, reg))
