@@ -48,6 +48,72 @@ func TestDiff_TableDiff(t *testing.T) {
 				},
 			},
 		},
+		// Attributes are specified and the same.
+		{
+			name: "no engine changes",
+			from: &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}, Attrs: []schema.Attr{&Engine{V: EngineInnoDB}}},
+			to:   &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}, Attrs: []schema.Attr{&Engine{V: EngineInnoDB}}},
+		},
+		// Attributes are specified and represent the same engine.
+		{
+			name: "no engine changes",
+			from: &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}, Attrs: []schema.Attr{&Engine{V: EngineInnoDB}}},
+			to:   &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}, Attrs: []schema.Attr{&Engine{V: "INNODB"}}},
+		},
+		// Attribute was dropped from the desired state, but the current state if the default (assumed one).
+		{
+			name: "no engine changes",
+			from: &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}, Attrs: []schema.Attr{&Engine{V: EngineInnoDB}}},
+			to:   &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}},
+		},
+		// Attribute was dropped from the desired state, but the current state if the default (explicitly specific as default).
+		{
+			name: "no engine changes",
+			from: &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}, Attrs: []schema.Attr{&Engine{V: EngineMyISAM, Default: true}}},
+			to:   &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}},
+		},
+		// The current state has no engine specified (unlikely case), and the desired state is set to the default (assumed one).
+		{
+			name: "no engine changes",
+			from: &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}},
+			to:   &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}, Attrs: []schema.Attr{&Engine{V: EngineInnoDB, Default: true}}},
+		},
+		// Attributes are specified, but they do not represent the same engine.
+		{
+			name: "engine changed",
+			from: &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}, Attrs: []schema.Attr{&Engine{V: EngineInnoDB}}},
+			to:   &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}, Attrs: []schema.Attr{&Engine{V: EngineMyISAM}}},
+			wantChanges: []schema.Change{
+				&schema.ModifyAttr{
+					From: &Engine{V: EngineInnoDB},
+					To:   &Engine{V: EngineMyISAM},
+				},
+			},
+		},
+		// The desired state has no engine specified, and the current state is not the default.
+		{
+			name: "engine changed",
+			from: &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}, Attrs: []schema.Attr{&Engine{V: EngineMyISAM}}},
+			to:   &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}},
+			wantChanges: []schema.Change{
+				&schema.ModifyAttr{
+					From: &Engine{V: EngineMyISAM},
+					To:   &Engine{V: EngineInnoDB, Default: true}, // Assume InnoDB is the default.
+				},
+			},
+		},
+		// The current state has no engine specified (unlikely case), and the desired state is not the default.
+		{
+			name: "engine changed",
+			from: &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}},
+			to:   &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}, Attrs: []schema.Attr{&Engine{V: EngineMyISAM}}},
+			wantChanges: []schema.Change{
+				&schema.ModifyAttr{
+					From: &Engine{V: EngineInnoDB, Default: true}, // Assume InnoDB is the default.
+					To:   &Engine{V: EngineMyISAM},
+				},
+			},
+		},
 		{
 			name: "add collation",
 			from: &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}, Attrs: []schema.Attr{&schema.Charset{V: "latin1"}}},
@@ -731,3 +797,77 @@ func TestDefaultDiff(t *testing.T) {
 	require.Len(t, changes, 1)
 	require.IsType(t, &schema.DropTable{}, changes[0])
 }
+
+func TestSkipChanges(t *testing.T) {
+	t.Run("DropSchema", func(t *testing.T) {
+		from, to := schema.NewRealm(schema.New("public")), schema.NewRealm()
+		changes, err := DefaultDiff.RealmDiff(from, to)
+		require.NoError(t, err)
+		require.Len(t, changes, 1)
+		changes, err = DefaultDiff.RealmDiff(from, to, schema.DiffSkipChanges(&schema.DropSchema{}))
+		require.NoError(t, err)
+		require.Empty(t, changes)
+	})
+
+	t.Run("DropTable", func(t *testing.T) {
+		from, to := schema.NewRealm(schema.New("public").AddTables(schema.NewTable("users"))), schema.NewRealm(schema.New("public"))
+		changes, err := DefaultDiff.RealmDiff(from, to)
+		require.NoError(t, err)
+		require.Len(t, changes, 1)
+		changes, err = DefaultDiff.SchemaDiff(from.Schemas[0], to.Schemas[0])
+		require.NoError(t, err)
+		require.Len(t, changes, 1)
+
+		changes, err = DefaultDiff.RealmDiff(from, to, schema.DiffSkipChanges(&schema.DropTable{}))
+		require.NoError(t, err)
+		require.Empty(t, changes)
+		changes, err = DefaultDiff.SchemaDiff(from.Schemas[0], to.Schemas[0], schema.DiffSkipChanges(&schema.DropTable{}))
+		require.NoError(t, err)
+		require.Empty(t, changes)
+	})
+
+	t.Run("ModifyTable", func(t *testing.T) {
+		from := schema.NewRealm(
+			schema.New("public").AddTables(
+				schema.NewTable("users").
+					AddColumns(schema.NewIntColumn("id", "int")).
+					AddIndexes(schema.NewIndex("users_id_idx").AddColumns(schema.NewIntColumn("id", "int"))),
+			),
+		)
+		to := schema.NewRealm(schema.New("public").AddTables(schema.NewTable("users")))
+		changes, err := DefaultDiff.RealmDiff(from, to)
+		require.NoError(t, err)
+		require.Len(t, changes, 1)
+		require.Len(t, changes[0].(*schema.ModifyTable).Changes, 2)
+		changes, err = DefaultDiff.SchemaDiff(from.Schemas[0], to.Schemas[0])
+		require.NoError(t, err)
+		require.Len(t, changes, 1)
+		require.Len(t, changes[0].(*schema.ModifyTable).Changes, 2)
+
+		changes, err = DefaultDiff.RealmDiff(from, to, schema.DiffSkipChanges(&schema.ModifyTable{}))
+		require.NoError(t, err)
+		require.Empty(t, changes)
+		changes, err = DefaultDiff.SchemaDiff(from.Schemas[0], to.Schemas[0], schema.DiffSkipChanges(&schema.ModifyTable{}))
+		require.NoError(t, err)
+		require.Empty(t, changes)
+
+		changes, err = DefaultDiff.RealmDiff(from, to, schema.DiffSkipChanges(&schema.DropColumn{}, &schema.DropIndex{}))
+		require.NoError(t, err)
+		require.Empty(t, changes)
+		changes, err = DefaultDiff.SchemaDiff(from.Schemas[0], to.Schemas[0], schema.DiffSkipChanges(&schema.DropColumn{}, &schema.DropIndex{}))
+		require.NoError(t, err)
+		require.Empty(t, changes)
+
+		// Ignore partial table changes.
+		changes, err = DefaultDiff.RealmDiff(from, to, schema.DiffSkipChanges(&schema.DropColumn{}))
+		require.NoError(t, err)
+		require.Len(t, changes, 1)
+		require.Len(t, changes[0].(*schema.ModifyTable).Changes, 1)
+		require.IsType(t, &schema.DropIndex{}, changes[0].(*schema.ModifyTable).Changes[0])
+		changes, err = DefaultDiff.SchemaDiff(from.Schemas[0], to.Schemas[0], schema.DiffSkipChanges(&schema.DropIndex{}))
+		require.NoError(t, err)
+		require.Len(t, changes, 1)
+		require.Len(t, changes[0].(*schema.ModifyTable).Changes, 1)
+		require.IsType(t, &schema.DropColumn{}, changes[0].(*schema.ModifyTable).Changes[0])
+	})
+}
