@@ -161,10 +161,11 @@ func (i *inspect) tables(ctx context.Context, realm *schema.Realm, opts *schema.
 	defer rows.Close()
 	for rows.Next() {
 		var (
-			autoinc                                             sql.NullInt64
-			tSchema, name, charset, collation, comment, options sql.NullString
+			defaultE                                                    sql.NullBool
+			autoinc                                                     sql.NullInt64
+			tSchema, name, charset, collation, comment, options, engine sql.NullString
 		)
-		if err := rows.Scan(&tSchema, &name, &charset, &collation, &autoinc, &comment, &options); err != nil {
+		if err := rows.Scan(&tSchema, &name, &charset, &collation, &autoinc, &comment, &options, &engine, &defaultE); err != nil {
 			return fmt.Errorf("scan table information: %w", err)
 		}
 		if !sqlx.ValidString(tSchema) || !sqlx.ValidString(name) {
@@ -196,6 +197,12 @@ func (i *inspect) tables(ctx context.Context, realm *schema.Realm, opts *schema.
 				V: options.String,
 			})
 		}
+		if sqlx.ValidString(engine) && defaultE.Valid {
+			t.Attrs = append(t.Attrs, &Engine{
+				V:       engine.String,
+				Default: defaultE.Bool,
+			})
+		}
 		if autoinc.Valid {
 			t.Attrs = append(t.Attrs, &AutoIncrement{
 				V: autoinc.Int64,
@@ -701,11 +708,15 @@ SELECT
 	t1.TABLE_COLLATION,
 	t1.AUTO_INCREMENT,
 	t1.TABLE_COMMENT,
-	t1.CREATE_OPTIONS
+	t1.CREATE_OPTIONS,
+	t1.ENGINE,
+	t3.SUPPORT = 'DEFAULT' AS DEFAULT_ENGINE
 FROM
 	INFORMATION_SCHEMA.TABLES AS t1
 	LEFT JOIN INFORMATION_SCHEMA.COLLATIONS AS t2
 	ON t1.TABLE_COLLATION = t2.COLLATION_NAME
+	LEFT JOIN INFORMATION_SCHEMA.ENGINES AS t3
+	ON t1.ENGINE = t3.ENGINE
 WHERE
 	TABLE_SCHEMA IN (%s)
 	AND TABLE_TYPE = 'BASE TABLE'
@@ -721,11 +732,15 @@ SELECT
 	t1.TABLE_COLLATION,
 	t1.AUTO_INCREMENT,
 	t1.TABLE_COMMENT,
-	t1.CREATE_OPTIONS
+	t1.CREATE_OPTIONS,
+	t1.ENGINE,
+	t3.SUPPORT = 'DEFAULT' AS DEFAULT_ENGINE
 FROM
 	INFORMATION_SCHEMA.TABLES AS t1
 	JOIN INFORMATION_SCHEMA.COLLATIONS AS t2
 	ON t1.TABLE_COLLATION = t2.COLLATION_NAME
+	LEFT JOIN INFORMATION_SCHEMA.ENGINES AS t3
+	ON t1.ENGINE = t3.ENGINE
 WHERE
 	TABLE_SCHEMA IN (%s)
 	AND TABLE_NAME IN (%s)
@@ -800,6 +815,13 @@ type (
 		S string
 	}
 
+	// Engine attribute describes the storage engine used to create a table.
+	Engine struct {
+		schema.Attr
+		V       string // InnoDB, MyISAM, etc.
+		Default bool   // The default engine used by the server.
+	}
+
 	// OnUpdate attribute for columns with "ON UPDATE CURRENT_TIMESTAMP" as a default.
 	OnUpdate struct {
 		schema.Attr
